import { Gui } from "Framework/Gui/Gui";
import { Graph } from "Framework/Gui/Graph";
import { TimeMeas } from "Framework/Time/TimeMeas";
import { LaplaceTransformFunction } from "./LaplaceTransformFunction";
import { LaplaceTransformInversionHuddleston } from "Framework/LaplaceTransform/LaplaceTransformInversionHuddleston";
import { LaplaceTransformInversionKoizumi } from "Framework/LaplaceTransform/LaplaceTransformInversionKoizumi";
import { LaplaceTransformInversionStehfest } from "Framework/LaplaceTransform/LaplaceTransformInversionStehfest";
import { LaplaceTransformInversionZakian } from "Framework/LaplaceTransform/LaplaceTransformInversionZakian";
import { LaplaceTransformInversionTalbot } from "Framework/LaplaceTransform/LaplaceTransformInversionTalbot";
import { ControlElement } from "../../shared/ControlElement";
var ControlAppLaplaceTransform = (function () {
    function ControlAppLaplaceTransform() {
    }
    ControlAppLaplaceTransform.prototype.Init = function () {
        this.FunctionSelected();
    };
    ControlAppLaplaceTransform.prototype.FunctionSelected = function () {
        var funcName = Gui.GetComboboxItem("FUNC");
        var htmlElemTransferFunc = document.getElementById("TRANSFER_FUNC");
        var htmlElemStepResponse = document.getElementById("STEP_RESPONSE");
        htmlElemTransferFunc.textContent = "$$" + ControlElement.GetTransferFunctionLatex(funcName) + "$$";
        MathJax.Hub.Queue(["Typeset", MathJax.Hub, htmlElemTransferFunc]);
        htmlElemStepResponse.textContent = "$$" + ControlElement.GetStepResponseLatex(funcName) + "$$";
        MathJax.Hub.Queue(["Typeset", MathJax.Hub, htmlElemStepResponse]);
    };
    ControlAppLaplaceTransform.prototype.NumericLaplaceInversion = function () {
        try {
            var funcName = Gui.GetComboboxItem("FUNC");
            var method1 = Gui.GetComboboxItem("METHOD_1");
            var method2 = Gui.GetComboboxItem("METHOD_2");
            var endTime = Gui.GetNumber("END_TIME");
            var timeStep = Gui.GetNumber("TIME_STEP");
            var numPoints = 1000;
            var idx = void 0;
            var func = this.GetFunction(funcName);
            var curveAna = [];
            for (idx = 0; idx < numPoints; idx++) {
                var t = endTime / (numPoints - 1) * idx;
                curveAna[idx] = [t, func.TimeFunctionStepResponse(t)];
            }
            var curveNameList = ["analytical"];
            var curveAxisList = [1];
            var curveColorList = ["#EE0000"];
            var curveWidthList = [4];
            var curveList = [curveAna];
            var timeMeas = new TimeMeas();
            if (method1.toLowerCase() != "none") {
                curveNameList.push(method1);
                curveAxisList.push(1);
                curveColorList.push("#00EE00");
                curveWidthList.push(1);
                timeMeas.Reset();
                var curve = this.GetCurve(method1, func, endTime, timeStep, numPoints);
                var time = timeMeas.Measure();
                curveList.push(curve);
                Gui.SetTableCell("TIME", 0, 0, "Time " + method1);
                Gui.SetTableCell("TIME", 1, 0, (time * 1000).toPrecision(4) + " ms");
            }
            else {
                Gui.SetTableCell("TIME", 0, 0, "");
                Gui.SetTableCell("TIME", 1, 0, "");
            }
            if (method2.toLowerCase() != "none") {
                curveNameList.push(method2);
                curveAxisList.push(1);
                curveColorList.push("#0000EE");
                curveWidthList.push(1);
                timeMeas.Reset();
                var curve = this.GetCurve(method2, func, endTime, timeStep, numPoints);
                var time = timeMeas.Measure();
                curveList.push(curve);
                Gui.SetTableCell("TIME", 0, 1, "Time " + method2);
                Gui.SetTableCell("TIME", 1, 1, (time * 1000).toPrecision(4) + " ms");
            }
            else {
                Gui.SetTableCell("TIME", 0, 1, "");
                Gui.SetTableCell("TIME", 1, 1, "");
            }
            Graph.Draw("DIV_GRAPH", "Step response", "Time", "Amplitude", "", curveNameList, curveAxisList, curveColorList, curveList, curveWidthList);
        }
        catch (err) {
            alert(err.message);
        }
    };
    ControlAppLaplaceTransform.prototype.GetFunction = function (func) {
        var K = Gui.GetNumber("PARAM_K");
        var T = Gui.GetNumber("PARAM_T");
        var D = Gui.GetNumber("PARAM_D");
        return LaplaceTransformFunction.Create(func, K, T, D);
    };
    ControlAppLaplaceTransform.prototype.GetCurve = function (methodName, func, endTime, timeStep, numPoints) {
        if (methodName.toLocaleLowerCase() == "deq") {
            return func.TimeFunctionDifferentialEquation(endTime, timeStep, numPoints);
        }
        else {
            var ilt = void 0;
            switch (methodName.toLowerCase()) {
                case "huddleston":
                    ilt = new LaplaceTransformInversionHuddleston();
                    break;
                case "koizumi":
                    ilt = new LaplaceTransformInversionKoizumi();
                    break;
                case "stehfest":
                    ilt = new LaplaceTransformInversionStehfest();
                    break;
                case "zakian":
                    ilt = new LaplaceTransformInversionZakian();
                    break;
                case "talbot":
                    ilt = new LaplaceTransformInversionTalbot();
                    break;
                default: throw new Error("invalid method: " + methodName);
            }
            var idx = void 0;
            var tArr = [];
            for (idx = 0; idx < numPoints; idx++)
                tArr[idx] = endTime / (numPoints - 1) * idx;
            var yArr = ilt.Calculate(func, tArr);
            var curve = [];
            for (idx = 0; idx < numPoints; idx++)
                curve[idx] = [tArr[idx], yArr[idx]];
            return curve;
        }
    };
    ControlAppLaplaceTransform.Inst = new ControlAppLaplaceTransform();
    return ControlAppLaplaceTransform;
}());
export { ControlAppLaplaceTransform };
//# sourceMappingURL=ControlAppLaplaceTransform.js.map