import { Complex } from "Framework/Math/Complex";
import { ControlElementD } from "../../shared/ControlElementD";
import { ControlElementDeadTime } from "../../shared/ControlElementDeadTime";
import { ControlElementI } from "../../shared/ControlElementI";
import { ControlElementP } from "../../shared/ControlElementP";
import { ControlElementPD1 } from "../../shared/ControlElementPD1";
import { ControlElementPT1 } from "../../shared/ControlElementPT1";
import { ControlElementPT2 } from "../../shared/ControlElementPT2";
var LaplaceTransformFunction = (function () {
    function LaplaceTransformFunction(ctrlElem) {
        this._res = new Complex();
        this._ctrlElem = ctrlElem;
    }
    LaplaceTransformFunction.Create = function (type, K, T, D) {
        switch (type) {
            case "D": return new LaplaceTransformFunction(new ControlElementD(K));
            case "DeadTime": return new LaplaceTransformFunction(new ControlElementDeadTime(T));
            case "I": return new LaplaceTransformFunction(new ControlElementI(T));
            case "P": return new LaplaceTransformFunction(new ControlElementP(K));
            case "PD1": return new LaplaceTransformFunction(new ControlElementPD1(K, T));
            case "PT1": return new LaplaceTransformFunction(new ControlElementPT1(K, T));
            case "PT2": return new LaplaceTransformFunction(new ControlElementPT2(K, T, D));
            default: throw new Error("unknown type: " + type);
        }
    };
    LaplaceTransformFunction.prototype.TimeFunctionStepResponse = function (t) {
        return this._ctrlElem.TimeFunctionStepResponse(t);
    };
    LaplaceTransformFunction.prototype.TransferFunction = function (s) {
        return this._ctrlElem.TransferFunction(s);
    };
    LaplaceTransformFunction.prototype.Function = function (s) {
        this._res.Copy(this.TransferFunction(s));
        this._res.Div(s);
        return this._res;
    };
    LaplaceTransformFunction.prototype.TimeFunctionDifferentialEquation = function (endTime, timeStep, numPoints) {
        var u = 1;
        var yArr = [];
        var t = 0;
        var idx = 0;
        var dt = endTime / (numPoints - 1);
        var tt = dt;
        this._ctrlElem.Initialize(timeStep);
        for (t = 0; t <= endTime; t += timeStep) {
            var y = this._ctrlElem.Calculate(u, timeStep);
            tt += timeStep;
            if (tt >= dt) {
                tt -= dt;
                yArr[idx] = [t, y];
                idx += 1;
            }
        }
        return yArr;
    };
    return LaplaceTransformFunction;
}());
export { LaplaceTransformFunction };
//# sourceMappingURL=LaplaceTransformFunction.js.map