var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { ControlElement } from "./ControlElement";
import { Complex } from "../../Framework/Math/Complex";
var ControlElementPD1 = (function (_super) {
    __extends(ControlElementPD1, _super);
    function ControlElementPD1(K, T) {
        var _this = _super.call(this) || this;
        _this._u_n1 = 0;
        _this._res = new Complex(0);
        if (T < 0)
            throw new Error("T must be greater then zero!");
        _this._K = K;
        _this._T = T;
        return _this;
    }
    ControlElementPD1.prototype.GetType = function () {
        return "PD1";
    };
    ControlElementPD1.prototype.GetLowestTimeConstant = function () {
        return this._T;
    };
    ControlElementPD1.prototype.Initialize = function (_dt) {
        this._u_n1 = 0;
    };
    ControlElementPD1.prototype.Calculate = function (u, dt) {
        var y = this._K * (u + (u - this._u_n1) * this._T / dt);
        this._u_n1 = u;
        return y;
    };
    ControlElementPD1.prototype.TimeFunctionStepResponse = function (t) {
        var n = 10;
        var d = n * Math.exp(-Math.PI * n * n * t * t);
        return this._K * (this._T * d + 1);
    };
    ControlElementPD1.prototype.TransferFunction = function (s) {
        this._res.Copy(s);
        this._res.MulR(this._T);
        this._res.AddR(1);
        this._res.MulR(this._K);
        return this._res;
    };
    ControlElementPD1.prototype.OptimizeGetNumParameters = function () {
        return 2;
    };
    ControlElementPD1.prototype.OptimizeSetParameters = function (params, startIdx) {
        this._K = params[startIdx + 0];
        this._T = params[startIdx + 1];
        if (this._T < 0)
            this._T = 0;
    };
    return ControlElementPD1;
}(ControlElement));
export { ControlElementPD1 };
//# sourceMappingURL=ControlElementPD1.js.map