import { Complex } from "../Math/Complex";
var LaplaceTransformInversionHuddleston = (function () {
    function LaplaceTransformInversionHuddleston(sigma, omega, n) {
        if (sigma === void 0) { sigma = 1; }
        if (omega === void 0) { omega = 200; }
        if (n === void 0) { n = 10000; }
        this._sigma = sigma;
        this._omega = omega;
        this._n = n;
    }
    LaplaceTransformInversionHuddleston.prototype.GetName = function () { return "Huddleston"; };
    LaplaceTransformInversionHuddleston.prototype.Calculate = function (F, tArr) {
        var yArr = [];
        var idx;
        var witi = new Complex(0);
        var wfti = new Complex(0);
        var fci = new Complex(0);
        var fcf = new Complex(0);
        for (idx = 0; idx < tArr.length; idx++) {
            var t = tArr[idx];
            if (t == 0) {
                yArr[idx] = 0;
                continue;
            }
            var sum = 0.0;
            var delta = this._omega / this._n;
            var wi = 0.0;
            var i = void 0;
            for (i = 0; i < this._n; i++) {
                var wf = wi + delta;
                fci.Set(this._sigma, wi);
                fcf.Set(this._sigma, wf);
                witi.Set(0, wi * t);
                witi.Exp();
                witi.Mul(F.Function(fci));
                wfti.Set(0, wf * t);
                wfti.Exp();
                wfti.Mul(F.Function(fcf));
                sum = sum + 0.5 * (wf - wi) * (witi.Real + wfti.Real);
                wi = wf;
            }
            yArr[idx] = sum * Math.exp(this._sigma * t) / Math.PI;
        }
        return yArr;
    };
    return LaplaceTransformInversionHuddleston;
}());
export { LaplaceTransformInversionHuddleston };
//# sourceMappingURL=LaplaceTransformInversionHuddleston.js.map