import { Complex } from "../Math/Complex";
import { MathEx } from "../Math/MathEx";
var LaplaceTransformInversionKoizumi = (function () {
    function LaplaceTransformInversionKoizumi(n) {
        if (n === void 0) { n = 2048; }
        if (n < 1)
            throw new Error("n < 1");
        this._n = n;
        this._coeff = [];
        this._coeff.length = n;
    }
    LaplaceTransformInversionKoizumi.prototype.GetName = function () { return "Koizumi"; };
    LaplaceTransformInversionKoizumi.prototype.Calculate = function (F, tArr) {
        var yArr = [];
        var T = MathEx.Max(tArr) * 4;
        var v1 = Math.PI / (2 * T);
        var v2 = 2.0 / T;
        var i;
        var z = new Complex(0);
        for (i = 0; i < this._n; i++) {
            z.Set(0, (1 - 2 * (i + 1)) * v1);
            this._coeff[i] = F.Function(z).Imag * v2;
        }
        for (i = 0; i < tArr.length; i++) {
            var t = tArr[i];
            if (t <= 0)
                yArr[i] = 0;
            else {
                var arg = t * v1;
                var ct = 2.0 * Math.cos(2.0 * arg);
                var c2 = 0.0;
                var c1 = this._coeff[this._n - 1];
                var k = void 0;
                for (k = this._n - 2; k >= 0; k--) {
                    var ck = ct * c1 - c2 + this._coeff[k];
                    c2 = c1;
                    c1 = ck;
                }
                yArr[i] = (c1 + c2) * Math.sin(arg);
            }
        }
        return yArr;
    };
    return LaplaceTransformInversionKoizumi;
}());
export { LaplaceTransformInversionKoizumi };
//# sourceMappingURL=LaplaceTransformInversionKoizumi.js.map