import { ControlElement } from "./ControlElement";
import { ControlElementP } from "./ControlElementP";
import { ControlElementI } from "./ControlElementI";
import { ControlElementD } from "./ControlElementD";
import { ControlElementPT1 } from "./ControlElementPT1";
import { ControlElementPT2 } from "./ControlElementPT2";
import { ControlElementPD1 } from "./ControlElementPD1";
import { ControlElementDeadTime } from "./ControlElementDeadTime";
var GuiControlElement = (function () {
    function GuiControlElement(pic, ctrlType, idx, hasParamK, hasParamT, hasParamD) {
        this.CTRL_TYPE_LIST = ["P", "I", "D", "PT1", "PT2", "PD1", "DeadTime"];
        this._hasParamK = false;
        this._hasParamT = false;
        this._hasParamD = false;
        this._k = 0;
        this._t = 0;
        this._d = 0;
        this._htmlElemTransferFunction = document.getElementById("LAPLACE_" + idx);
        this._pic = pic;
        this._ctrlType = ctrlType;
        this._idx = idx;
        this._hasParamK = hasParamK;
        this._hasParamT = hasParamT;
        this._hasParamD = hasParamD;
        if (ctrlType == "NONE")
            this._htmlElemCtrl = null;
        else
            this._htmlElemCtrl = pic.contentDocument.getElementById(ctrlType + "_CTRL_" + idx);
        this._htmlElemParamK = document.getElementById("PARAM_K_" + idx);
        this._htmlElemParamT = document.getElementById("PARAM_T_" + idx);
        this._htmlElemParamD = document.getElementById("PARAM_D_" + idx);
    }
    Object.defineProperty(GuiControlElement.prototype, "CtrlType", {
        get: function () { return this._ctrlType; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(GuiControlElement.prototype, "HasParamK", {
        get: function () { return this._hasParamK; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(GuiControlElement.prototype, "HasParamT", {
        get: function () { return this._hasParamT; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(GuiControlElement.prototype, "HasParamD", {
        get: function () { return this._hasParamD; },
        enumerable: true,
        configurable: true
    });
    GuiControlElement.Create = function (ctrlType, idx, pic) {
        switch (ctrlType) {
            case "P": return new GuiControlElement(pic, "P", idx, true, false, false);
            case "I": return new GuiControlElement(pic, "I", idx, false, true, false);
            case "D": return new GuiControlElement(pic, "D", idx, true, false, false);
            case "PT1": return new GuiControlElement(pic, "PT1", idx, true, true, false);
            case "PT2": return new GuiControlElement(pic, "PT2", idx, true, true, true);
            case "PD1": return new GuiControlElement(pic, "PD1", idx, true, true, false);
            case "DeadTime": return new GuiControlElement(pic, "DeadTime", idx, false, true, false);
            case "NONE":
            default: return new GuiControlElement(pic, "NONE", idx, false, false, false);
        }
    };
    GuiControlElement.prototype.CreateControl = function () {
        this.TakeParams();
        switch (this.CtrlType) {
            case "P": return new ControlElementP(this._k);
            case "I": return new ControlElementI(this._t);
            case "D": return new ControlElementD(this._k);
            case "PT1": return new ControlElementPT1(this._k, this._t);
            case "PT2": return new ControlElementPT2(this._k, this._t, this._d);
            case "PD1": return new ControlElementPD1(this._k, this._t);
            case "DeadTime": return new ControlElementDeadTime(this._t);
            default:
                throw new TypeError("CreateControl(): unknown type " + this.CtrlType);
        }
    };
    GuiControlElement.prototype.Show = function () {
        var i;
        for (i = 0; i < this.CTRL_TYPE_LIST.length; i++) {
            var ctrlType = this.CTRL_TYPE_LIST[i];
            this._pic.contentDocument.getElementById(ctrlType + "_CTRL_" + this._idx).style.display = "none";
        }
        if (this._htmlElemCtrl != null)
            this._htmlElemCtrl.style.display = "initial";
        if (this._htmlElemTransferFunction != null) {
            this._htmlElemTransferFunction.textContent = "$$" + ControlElement.GetTransferFunctionLatex(this._ctrlType) + "$$";
            MathJax.Hub.Queue(["Typeset", MathJax.Hub, this._htmlElemTransferFunction]);
        }
    };
    GuiControlElement.prototype.ShowParams = function () {
        this._htmlElemParamK.style.display = this.HasParamK ? "initial" : "none";
        this._htmlElemParamT.style.display = this.HasParamT ? "initial" : "none";
        this._htmlElemParamD.style.display = this.HasParamD ? "initial" : "none";
    };
    GuiControlElement.prototype.TakeParams = function () {
        if (this.HasParamK) {
            if (this._htmlElemParamK.value.length == 0)
                throw new Error("row " + this._idx + " control element " + this.CtrlType + " invalid parameter K");
            this._k = Number(this._htmlElemParamK.value);
        }
        if (this.HasParamT) {
            if (this._htmlElemParamT.value.length == 0)
                throw new Error("row " + this._idx + " control element " + this.CtrlType + " invalid parameter T");
            this._t = Number(this._htmlElemParamT.value);
        }
        if (this.HasParamD) {
            if (this._htmlElemParamD.value.length == 0)
                throw new Error("row " + this._idx + " control element " + this.CtrlType + " invalid parameter D");
            this._d = Number(this._htmlElemParamD.value);
        }
        this.UpdateParamsInPic();
    };
    GuiControlElement.prototype.UpdateParamsInPic = function () {
        if (this.HasParamK)
            this._pic.contentDocument.getElementById(this.CtrlType + "_CTRL_" + this._idx + "_K").textContent = "K = " + this._k.toPrecision(3);
        if (this.HasParamT)
            this._pic.contentDocument.getElementById(this.CtrlType + "_CTRL_" + this._idx + "_T").textContent = "T = " + this._t.toPrecision(3);
        if (this.HasParamD)
            this._pic.contentDocument.getElementById(this.CtrlType + "_CTRL_" + this._idx + "_D").textContent = "D = " + this._d.toPrecision(3);
    };
    GuiControlElement.prototype.UpdateParamsInInputs = function () {
        if (this.HasParamK)
            this._htmlElemParamK.value = this._k.toPrecision(5);
        if (this.HasParamT)
            this._htmlElemParamT.value = this._t.toPrecision(5);
        if (this.HasParamD)
            this._htmlElemParamD.value = this._d.toPrecision(5);
    };
    GuiControlElement.prototype.GetNumParams = function () {
        var cnt = 0;
        if (this.HasParamK)
            cnt++;
        if (this.HasParamT)
            cnt++;
        if (this.HasParamD)
            cnt++;
        return cnt;
    };
    GuiControlElement.prototype.ShowParamVals = function (params, paramIdx) {
        if (this.HasParamK)
            this._k = params[paramIdx++];
        if (this.HasParamT)
            this._t = params[paramIdx++];
        if (this.HasParamD)
            this._d = params[paramIdx++];
        this.UpdateParamsInPic();
        this.UpdateParamsInInputs();
    };
    return GuiControlElement;
}());
export { GuiControlElement };
//# sourceMappingURL=GuiControlElement.js.map