var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { Gui } from "Framework/Gui/Gui";
import { Graph } from "Framework/Gui/Graph";
import { Derivative } from "Framework/Math/Derivative";
import { ControlApp } from "./ControlApp";
import { ControlLoop } from "./ControlLoop";
import { ControlSimulation } from "./ControlSimulation";
import { ControlPID } from "./ControlPID";
var ControlApp_ControlLoopStepResponse = (function (_super) {
    __extends(ControlApp_ControlLoopStepResponse, _super);
    function ControlApp_ControlLoopStepResponse() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ControlApp_ControlLoopStepResponse.prototype.ControlLoopStepResponse = function () {
        try {
            var pidKp = Gui.GetNumber("PID_KP");
            var pidKi = Gui.GetNumber("PID_KI");
            var pidKd = Gui.GetNumber("PID_KD");
            var pidUMin = Gui.GetNumber("PID_U_MIN");
            var pidUMax = Gui.GetNumber("PID_U_MAX");
            var show1stDerivative = Gui.IsCheckedWithDefault("SHOW_DERIVATIVE_1", false);
            var show2ndDerivative = Gui.IsCheckedWithDefault("SHOW_DERIVATIVE_2", false);
            var stepHeight = Gui.GetNumber("STEP_HEIGHT");
            var stepTime = Gui.GetNumber("STEP_TIME");
            var endTime = Gui.GetNumber("END_TIME");
            this.SetControlParamInGuiPic("PID_CTRL_1_KP", "Kp = " + pidKp.toPrecision(3));
            this.SetControlParamInGuiPic("PID_CTRL_1_KI", "Ki = " + pidKi.toPrecision(3));
            this.SetControlParamInGuiPic("PID_CTRL_1_KD", "Kd = " + pidKd.toPrecision(3));
            var ctrlPath = this.CreateControlPathFromGui();
            var ctrlPid = new ControlPID(pidKp, pidKi, pidKd, pidUMin, pidUMax);
            var ctrlLoop = new ControlLoop(ctrlPid, ctrlPath);
            var ctrlSim = new ControlSimulation(ctrlLoop);
            var curveW = void 0;
            var curveU = void 0;
            var curveY = void 0;
            var maxY = void 0;
            _a = ctrlSim.StepResponseControlLoop(stepHeight, stepTime, endTime), curveW = _a[0], curveU = _a[1], curveY = _a[2], maxY = _a[3];
            var curveNameList = ["u(t)", "w(t)", "y(t)"];
            var yAxisList = [2, 1, 1];
            var curveColorList = ["#00FF00", "#0000FF", "#FF0000"];
            var curveDataList = [curveU, curveW, curveY];
            if (show1stDerivative) {
                var curveDy1 = Derivative.DerivativeEx(curveY);
                curveNameList.push("1st derivative y(t)");
                yAxisList.push(1);
                curveColorList.push("#9400D3");
                curveDataList.push(curveDy1);
            }
            if (show2ndDerivative) {
                var curveDy1 = Derivative.DerivativeEx(curveY);
                var curveDy2 = Derivative.DerivativeEx(curveDy1);
                curveNameList.push("2nd derivative y(t)");
                yAxisList.push(1);
                curveColorList.push("#4B0082");
                curveDataList.push(curveDy2);
            }
            Graph.Draw("DIV_GRAPH", "Step response", "Time", "Amplitude", "Amplitude", curveNameList, yAxisList, curveColorList, curveDataList);
            var overshotPercent = (maxY - stepHeight) / stepHeight * 100;
            document.getElementById("OVERSHOT").textContent = "Überschwingen: " + overshotPercent.toFixed(2) + "%";
        }
        catch (err) {
            alert(err.message);
        }
        var _a;
    };
    return ControlApp_ControlLoopStepResponse;
}(ControlApp));
export { ControlApp_ControlLoopStepResponse };
//# sourceMappingURL=ControlApp_ControlLoopStepResponse.js.map