var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { Gui } from "Framework/Gui/Gui";
import { ControlApp_ControlLoopStepResponse } from "./ControlApp_ControlLoopStepResponse";
import { ControlLoop } from "./ControlLoop";
import { ControlPID } from "./ControlPID";
import { ControllerOptimizationLocalVariation } from "./ControllerOptimizationLocalVariation";
import { ControllerOptimizationDifferentialEvolution } from "./ControllerOptimizationDifferentialEvolution";
var ControlApp_ControlLoopTuningIteration = (function (_super) {
    __extends(ControlApp_ControlLoopTuningIteration, _super);
    function ControlApp_ControlLoopTuningIteration() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ControlApp_ControlLoopTuningIteration.prototype.ControlLoopTuningLocalVariation = function () {
        try {
            var pidKp = Gui.GetNumber("PID_KP");
            var pidKi = Gui.GetNumber("PID_KI");
            var pidKd = Gui.GetNumber("PID_KD");
            var pidUMin = Gui.GetNumber("PID_U_MIN");
            var pidUMax = Gui.GetNumber("PID_U_MAX");
            var stepHeight = Gui.GetNumber("STEP_HEIGHT");
            var stepTime = Gui.GetNumber("STEP_TIME");
            var endTime = Gui.GetNumber("END_TIME");
            var maxNumSteps = Gui.GetNumber("MAX_NUM_STEPS");
            var maxOvershotPercent = Gui.GetNumber("MAX_OVERSHOT");
            var weightingCtrlOut = Gui.GetNumber("WEIGHTING_CTRL_OUT");
            var weightingOvershot = Gui.GetNumber("WEIGHTING_OVERSHOT");
            var ctrlPath = this.CreateControlPathFromGui();
            var ctrlPid = new ControlPID(pidKp, pidKi, pidKd, pidUMin, pidUMax);
            var ctrlLoop = new ControlLoop(ctrlPid, ctrlPath);
            var ctrlOpti = new ControllerOptimizationLocalVariation(ctrlLoop, ctrlPid);
            var startParams = [pidKp, pidKi, pidKd];
            var startDelta = [pidKp + 10, pidKi + 10, pidKd + 10];
            var maxOvershot = Math.abs(maxOvershotPercent) / 100 * stepHeight;
            var params = ctrlOpti.Optimize(startParams, startDelta, maxNumSteps, stepHeight, stepTime, endTime, maxOvershot, weightingCtrlOut, weightingOvershot);
            Gui.Set("PID_KP", params[0]);
            Gui.Set("PID_KI", params[1]);
            Gui.Set("PID_KD", params[2]);
            this.SetControlParamInGuiPic("PID_CTRL_1_KP", "Kp = " + params[0].toPrecision(3));
            this.SetControlParamInGuiPic("PID_CTRL_1_KI", "Ki = " + params[1].toPrecision(3));
            this.SetControlParamInGuiPic("PID_CTRL_1_KD", "Kd = " + params[2].toPrecision(3));
        }
        catch (err) {
            alert(err.message);
        }
        this.ControlLoopStepResponse();
    };
    ControlApp_ControlLoopTuningIteration.prototype.ControlLoopTuningDiffEvo = function () {
        try {
            var pidKp = Gui.GetNumber("PID_KP");
            var pidKi = Gui.GetNumber("PID_KI");
            var pidKd = Gui.GetNumber("PID_KD");
            var pidUMin = Gui.GetNumber("PID_U_MIN");
            var pidUMax = Gui.GetNumber("PID_U_MAX");
            var stepHeight = Gui.GetNumber("STEP_HEIGHT");
            var stepTime = Gui.GetNumber("STEP_TIME");
            var endTime = Gui.GetNumber("END_TIME");
            var maxNumSteps = Gui.GetNumber("MAX_NUM_STEPS");
            var maxOvershotPercent = Gui.GetNumber("MAX_OVERSHOT");
            var weightingCtrlOut = Gui.GetNumber("WEIGHTING_CTRL_OUT");
            var weightingOvershot = Gui.GetNumber("WEIGHTING_OVERSHOT");
            var ctrlPath = this.CreateControlPathFromGui();
            var ctrlPid = new ControlPID(pidKp, pidKi, pidKd, pidUMin, pidUMax);
            var ctrlLoop = new ControlLoop(ctrlPid, ctrlPath);
            var ctrlOpti = new ControllerOptimizationDifferentialEvolution(ctrlLoop, ctrlPid);
            var maxOvershot = Math.abs(maxOvershotPercent) / 100 * stepHeight;
            var dither = true;
            var params = ctrlOpti.Optimize(0, 100, dither, maxNumSteps, stepHeight, stepTime, endTime, maxOvershot, weightingCtrlOut, weightingOvershot);
            Gui.Set("PID_KP", params[0]);
            Gui.Set("PID_KI", params[1]);
            Gui.Set("PID_KD", params[2]);
            this.SetControlParamInGuiPic("PID_CTRL_1_KP", "Kp = " + params[0].toPrecision(3));
            this.SetControlParamInGuiPic("PID_CTRL_1_KI", "Ki = " + params[1].toPrecision(3));
            this.SetControlParamInGuiPic("PID_CTRL_1_KD", "Kd = " + params[2].toPrecision(3));
        }
        catch (err) {
            alert(err.message);
        }
        this.ControlLoopStepResponse();
    };
    return ControlApp_ControlLoopTuningIteration;
}(ControlApp_ControlLoopStepResponse));
export { ControlApp_ControlLoopTuningIteration };
//# sourceMappingURL=ControlApp_ControlLoopTuningIteration.js.map