var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { Gui } from "Framework/Gui/Gui";
import { Graph } from "Framework/Gui/Graph";
import { ControlApp } from "./ControlApp";
import { ControlTools } from "./ControlTools";
import { ControlLoop } from "./ControlLoop";
import { ControlSimulation } from "./ControlSimulation";
import { ControlPID } from "./ControlPID";
import { ControlComparator } from "./ControlComparator";
var ControlApp_ControlLoopTuningOscillation = (function (_super) {
    __extends(ControlApp_ControlLoopTuningOscillation, _super);
    function ControlApp_ControlLoopTuningOscillation() {
        var _this = _super.call(this) || this;
        _this._ku = 0;
        _this._pu = 0;
        return _this;
    }
    ControlApp_ControlLoopTuningOscillation.prototype.ControlLoopTuningOscillation = function () {
        try {
            var comparatorTh = Gui.GetNumber("COMPARATOR_TH");
            var comparatorUMin = Gui.GetNumber("COMPARATOR_U_MIN");
            var comparatorUMax = Gui.GetNumber("COMPARATOR_U_MAX");
            var endTime = Gui.GetNumber("END_TIME");
            this.SetControlParamInGuiPic("COMPARATOR_TH", "Th = " + comparatorTh);
            var ctrlPath = this.CreateControlPathFromGui();
            var ctrlComparator = new ControlComparator(comparatorTh, comparatorUMin, comparatorUMax);
            var ctrlLoop = new ControlLoop(ctrlComparator, ctrlPath);
            var ctrlSim = new ControlSimulation(ctrlLoop);
            var curveW = void 0;
            var curveU = void 0;
            var curveY = void 0;
            _a = ctrlSim.StepResponseControlLoop(0, 0, endTime), curveW = _a[0], curveU = _a[1], curveY = _a[2];
            var au = ctrlComparator.Amplitude;
            var ay = ctrlLoop.Amplitude;
            var p = ctrlComparator.Period;
            this._pu = p;
            this._ku = 4 * au / (Math.PI * ay);
            Gui.SetTableCell("TABLE_PATH_PARAM", 1, 2, au.toPrecision(4));
            Gui.SetTableCell("TABLE_PATH_PARAM", 2, 2, ay.toPrecision(4));
            Gui.SetTableCell("TABLE_PATH_PARAM", 3, 2, p.toPrecision(4));
            Gui.SetTableCell("TABLE_PATH_PARAM", 4, 2, this._ku.toPrecision(4));
            Gui.SetTableCell("TABLE_PATH_PARAM", 5, 2, this._pu.toPrecision(4));
            var kp = void 0;
            var ki = void 0;
            var kd = void 0;
            _b = ControlTools.GetPidParamsZieglerNichols(this._pu, this._ku), kp = _b[0], ki = _b[1], kd = _b[2];
            this.ShowPidParams(1, kp, ki, kd);
            _c = ControlTools.GetPidParamsLittleOvershot(this._pu, this._ku), kp = _c[0], ki = _c[1], kd = _c[2];
            this.ShowPidParams(2, kp, ki, kd);
            _d = ControlTools.GetPidParamsNoOvershot(this._pu, this._ku), kp = _d[0], ki = _d[1], kd = _d[2];
            this.ShowPidParams(3, kp, ki, kd);
            Graph.Draw("DIV_GRAPH", "PID tuning relay method", "Time", "Amplitude", "Amplitude", ["u(t)", "y(t)"], [1, 1], ["#0000FF", "#FF0000"], [curveU, curveY]);
            this.ControlLoopTuningOscillationParamMethodSelected();
        }
        catch (err) {
            alert(err.message);
        }
        var _a, _b, _c, _d;
    };
    ControlApp_ControlLoopTuningOscillation.prototype.ShowPidParams = function (row, kp, ki, kd) {
        Gui.SetTableCell("TABLE_PID_PARAMS", row, 1, kp.toPrecision(4));
        Gui.SetTableCell("TABLE_PID_PARAMS", row, 3, ki.toPrecision(4));
        Gui.SetTableCell("TABLE_PID_PARAMS", row, 2, (kp / ki).toPrecision(4));
        Gui.SetTableCell("TABLE_PID_PARAMS", row, 5, kd.toPrecision(4));
        Gui.SetTableCell("TABLE_PID_PARAMS", row, 4, (kd / kp).toPrecision(4));
    };
    ControlApp_ControlLoopTuningOscillation.prototype.ControlLoopTuningOscillationParamMethodSelected = function () {
        var kp;
        var ki;
        var kd;
        var pidParams = Gui.GetString("PID_PARAMS");
        switch (pidParams) {
            case "L":
                _a = ControlTools.GetPidParamsLittleOvershot(this._pu, this._ku), kp = _a[0], ki = _a[1], kd = _a[2];
                break;
            case "N":
                _b = ControlTools.GetPidParamsNoOvershot(this._pu, this._ku), kp = _b[0], ki = _b[1], kd = _b[2];
                break;
            case "Z":
            default:
                _c = ControlTools.GetPidParamsZieglerNichols(this._pu, this._ku), kp = _c[0], ki = _c[1], kd = _c[2];
                break;
        }
        Gui.Set("PID_KP", kp);
        Gui.Set("PID_KI", ki);
        Gui.Set("PID_KD", kd);
        var _a, _b, _c;
    };
    ControlApp_ControlLoopTuningOscillation.prototype.ControlLoopTuningOscillationStepResponse = function () {
        try {
            var pidKp = Gui.GetNumber("PID_KP");
            var pidKi = Gui.GetNumber("PID_KI");
            var pidKd = Gui.GetNumber("PID_KD");
            var pidUMin = Gui.GetNumber("PID_U_MIN");
            var pidUMax = Gui.GetNumber("PID_U_MAX");
            var stepHeight = Gui.GetNumber("STEP_HEIGHT");
            var stepTime = Gui.GetNumber("STEP_TIME");
            var endTime = Gui.GetNumber("END_TIME_STEP_RESPONSE");
            var ctrlPath = this.CreateControlPathFromGui();
            var ctrlPid = new ControlPID(pidKp, pidKi, pidKd, pidUMin, pidUMax);
            var ctrlLoop = new ControlLoop(ctrlPid, ctrlPath);
            var ctrlSim = new ControlSimulation(ctrlLoop);
            var curveW = void 0;
            var curveU = void 0;
            var curveY = void 0;
            var maxY = void 0;
            _a = ctrlSim.StepResponseControlLoop(stepHeight, stepTime, endTime), curveW = _a[0], curveU = _a[1], curveY = _a[2], maxY = _a[3];
            Graph.Draw("DIV_GRAPH_STEP_RESPONSE", "Step response", "Time", "Amplitude", "Amplitude", ["u(t)", "w(t)", "y(t)"], [2, 1, 1], ["#00FF00", "#0000FF", "#FF0000"], [curveU, curveW, curveY]);
            var overshotPercent = (maxY - stepHeight) / stepHeight * 100;
            document.getElementById("OVERSHOT").textContent = "Überschwingen: " + overshotPercent.toFixed(2) + "%";
        }
        catch (err) {
            alert(err.message);
        }
        var _a;
    };
    return ControlApp_ControlLoopTuningOscillation;
}(ControlApp));
export { ControlApp_ControlLoopTuningOscillation };
//# sourceMappingURL=ControlApp_ControlLoopTuningOscillation.js.map