var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { ControlElement } from "./ControlElement";
import { Complex } from "Framework/Math/Complex";
var ControlLoop = (function (_super) {
    __extends(ControlLoop, _super);
    function ControlLoop(controller, controlPath) {
        var _this = _super.call(this) || this;
        _this._u = 0;
        _this._y_n1 = 0;
        _this._yMin = 0;
        _this._yMax = 0;
        _this._res = new Complex(0);
        _this._tmp = new Complex(0);
        _this._controller = controller;
        _this._controlPath = controlPath;
        return _this;
    }
    Object.defineProperty(ControlLoop.prototype, "U", {
        get: function () { return this._u; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ControlLoop.prototype, "YMin", {
        get: function () { return this._yMin; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ControlLoop.prototype, "YMax", {
        get: function () { return this._yMax; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ControlLoop.prototype, "Amplitude", {
        get: function () { return (this._yMax - this._yMin) / 2; },
        enumerable: true,
        configurable: true
    });
    ControlLoop.prototype.GetType = function () {
        return "ControlLoop";
    };
    ControlLoop.prototype.GetLowestTimeConstant = function () {
        var t1 = this._controller.GetLowestTimeConstant();
        var t2 = this._controlPath.GetLowestTimeConstant();
        if (t1 == -1)
            return t2;
        if (t2 == -1)
            return t1;
        return Math.min(t1, t2);
    };
    ControlLoop.prototype.Initialize = function (dt) {
        this._u = 0;
        this._y_n1 = 0;
        this._yMin = 0;
        this._yMax = 0;
        this._direction = 1;
        this._controller.Initialize(dt);
        this._controlPath.Initialize(dt);
    };
    ControlLoop.prototype.Calculate = function (w, dt) {
        var e = w - this._y_n1;
        var u = this._controller.Calculate(e, dt);
        var y = this._controlPath.Calculate(u, dt);
        if ((this._direction == -1) && (this._y_n1 < y)) {
            this._yMin = this._y_n1;
            this._direction = 1;
        }
        if ((this._direction == 1) && (this._y_n1 > y)) {
            this._yMax = this._y_n1;
            this._direction = -1;
        }
        this._y_n1 = y;
        this._u = u;
        return y;
    };
    ControlLoop.prototype.TimeFunctionStepResponse = function (_t) {
        return 0;
    };
    ControlLoop.prototype.TransferFunction = function (s) {
        this._tmp.Copy(this._controller.TransferFunction(s));
        this._tmp.Mul(this._controlPath.TransferFunction(s));
        this._res.Copy(this._tmp);
        this._tmp.AddR(1);
        this._res.Div(this._tmp);
        return this._res;
    };
    ControlLoop.prototype.OptimizeGetNumParameters = function () {
        return this._controller.OptimizeGetNumParameters() + this._controlPath.OptimizeGetNumParameters();
    };
    ControlLoop.prototype.OptimizeSetParameters = function (params, startIdx) {
        this._controller.OptimizeSetParameters(params, startIdx);
        this._controlPath.OptimizeSetParameters(params, startIdx + this._controller.OptimizeGetNumParameters());
    };
    return ControlLoop;
}(ControlElement));
export { ControlLoop };
//# sourceMappingURL=ControlLoop.js.map