var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { ControlElement } from "./ControlElement";
import { Complex } from "Framework/Math/Complex";
var ControlPath = (function (_super) {
    __extends(ControlPath, _super);
    function ControlPath(controlElementList) {
        var _this = _super.call(this) || this;
        _this._res = new Complex(0);
        if (controlElementList) {
            _this._controlElementList = controlElementList;
        }
        else {
            _this._controlElementList = [];
        }
        return _this;
    }
    ControlPath.prototype.Add = function (controlelement) {
        this._controlElementList.push(controlelement);
    };
    ControlPath.prototype.GetType = function () {
        return "ControlPath";
    };
    ControlPath.prototype.GetLowestTimeConstant = function () {
        var t = -1;
        var idx;
        for (idx = 0; idx < this._controlElementList.length; idx++) {
            var tt = this._controlElementList[idx].GetLowestTimeConstant();
            if (t == -1) {
                t = tt;
            }
            else {
                if ((tt != -1) && (tt < t))
                    t = tt;
            }
        }
        return t;
    };
    ControlPath.prototype.Initialize = function (dt) {
        var idx;
        for (idx = 0; idx < this._controlElementList.length; idx++) {
            this._controlElementList[idx].Initialize(dt);
        }
    };
    ControlPath.prototype.Calculate = function (u, dt) {
        var idx;
        var y = u;
        for (idx = 0; idx < this._controlElementList.length; idx++) {
            y = this._controlElementList[idx].Calculate(y, dt);
        }
        return y;
    };
    ControlPath.prototype.TimeFunctionStepResponse = function (_t) {
        return 0;
    };
    ControlPath.prototype.TransferFunction = function (s) {
        if (this._controlElementList.length < 1) {
            this._res.Set(0, 0);
        }
        else {
            var idx = void 0;
            this._res.Copy(this._controlElementList[0].TransferFunction(s));
            for (idx = 1; idx < this._controlElementList.length; idx++)
                this._res.Mul(this._controlElementList[idx].TransferFunction(s));
        }
        return this._res;
    };
    ControlPath.prototype.OptimizeGetNumParameters = function () {
        var numParams = 0;
        var idx;
        for (idx = 0; idx < this._controlElementList.length; idx++) {
            numParams += this._controlElementList[idx].OptimizeGetNumParameters();
        }
        return numParams;
    };
    ControlPath.prototype.OptimizeSetParameters = function (params, startIdx) {
        var idx;
        var paramIdx = startIdx;
        for (idx = 0; idx < this._controlElementList.length; idx++) {
            this._controlElementList[idx].OptimizeSetParameters(params, paramIdx);
            paramIdx += this._controlElementList[idx].OptimizeGetNumParameters();
        }
    };
    return ControlPath;
}(ControlElement));
export { ControlPath };
//# sourceMappingURL=ControlPath.js.map