import { Derivative } from "Framework/Math/Derivative";
import { MathEx } from "Framework/Math/MathEx";
var ControlTools = (function () {
    function ControlTools() {
    }
    ControlTools.CalcStepParams = function (uArr) {
        var stepHeight = 0;
        var endTime = 0;
        var stepTime = 0;
        var idx;
        for (idx = 0; idx < uArr.length; idx++) {
            if (uArr[idx][0] > endTime)
                endTime = uArr[idx][0];
            if (uArr[idx][1] > stepHeight)
                stepHeight = uArr[idx][1];
        }
        for (idx = 0; idx < uArr.length; idx++) {
            if (uArr[idx][1] > stepHeight / 2) {
                stepTime = uArr[idx][0];
                break;
            }
        }
        return [stepHeight, stepTime, endTime];
    };
    ControlTools.GetInflectionTangent = function (f, startX) {
        var X = 0;
        var Y = 1;
        var d = Derivative.DerivativeEx(f);
        var dd = Derivative.DerivativeEx(d);
        var idx;
        for (idx = 1; idx < f.length; idx++) {
            if (f[idx][X] > startX)
                break;
        }
        for (; idx < f.length; idx++) {
            if (MathEx.Sign(dd[idx][Y]) != MathEx.Sign(dd[idx - 1][Y]))
                break;
        }
        if (idx == f.length)
            return [0, 0];
        var m = d[idx][Y];
        var n = f[idx][Y] - m * f[idx][X];
        return [m, n];
    };
    ControlTools.GetPidParamsZieglerNichols = function (tu, ku) {
        var kp = 0.6 * ku;
        var tn = 0.5 * tu;
        var tv = 0.125 * tu;
        return [kp, kp / tn, kp * tv];
    };
    ControlTools.GetPidParamsLittleOvershot = function (tu, ku) {
        var kp = 0.33 * ku;
        var tn = 0.5 * tu;
        var tv = 0.333 * tu;
        return [kp, kp / tn, kp * tv];
    };
    ControlTools.GetPidParamsNoOvershot = function (tu, ku) {
        var kp = 0.2 * ku;
        var tn = 0.5 * tu;
        var tv = 0.333 * tu;
        return [kp, kp / tn, kp * tv];
    };
    ControlTools.ParseMeasDataString = function (measDataStr) {
        var uArr = [];
        var yArr = [];
        var measDataStrList = measDataStr.split("\n");
        var lineNum;
        for (lineNum = 0; lineNum < measDataStrList.length; lineNum++) {
            var line = measDataStrList[lineNum].trim();
            if (line.length > 0) {
                var valStrList = line.split(",");
                var t = Number(valStrList[0]);
                var u = Number(valStrList[1]);
                var y = Number(valStrList[2]);
                uArr.push([t, u]);
                yArr.push([t, y]);
            }
        }
        return [uArr, yArr];
    };
    return ControlTools;
}());
export { ControlTools };
//# sourceMappingURL=ControlTools.js.map