import { ControlOptimizationError } from "./ControlOptimizationError";
var ControllerOptimizationLocalVariation = (function () {
    function ControllerOptimizationLocalVariation(controlLoop, controlOptimization) {
        this._NUM_TIME_STEPS = 20000;
        this._controlLoop = controlLoop;
        this._controlOptimization = controlOptimization;
    }
    ControllerOptimizationLocalVariation.prototype.Optimize = function (startParams, startDelta, maxNumSteps, stepHeight, stepTime, endTime, maxOvershot, weightingCtrlOut, weightingOvershot) {
        this._calcError = new ControlOptimizationError(this._controlLoop, stepHeight, maxOvershot, weightingCtrlOut, weightingOvershot);
        var paramsCount = this._controlOptimization.OptimizeGetNumParameters();
        var params = [];
        var delta = [];
        var idx;
        for (idx = 0; idx < paramsCount; idx++) {
            params[idx] = startParams[idx];
            delta[idx] = startDelta[idx];
        }
        var step = 0;
        var err;
        err = this.Evaluate(params, stepHeight, stepTime, endTime);
        while (step < maxNumSteps) {
            var better = false;
            for (idx = 0; idx < paramsCount; idx++) {
                var errAdd = void 0;
                var errSub = void 0;
                var p = params[idx];
                params[idx] = p + delta[idx];
                errAdd = this.Evaluate(params, stepHeight, stepTime, endTime);
                if (p > delta[idx]) {
                    params[idx] = p - delta[idx];
                    errSub = this.Evaluate(params, stepHeight, stepTime, endTime);
                }
                else
                    errSub = err + 1;
                if ((errAdd < err) || (errSub < err)) {
                    if (errAdd <= errSub) {
                        err = errAdd;
                        params[idx] = p + delta[idx];
                    }
                    else {
                        err = errSub;
                        params[idx] = p - delta[idx];
                    }
                    better = true;
                }
                else {
                    params[idx] = p;
                }
            }
            if (!better) {
                var smallEnough = true;
                for (idx = 0; idx < paramsCount; idx++) {
                    delta[idx] /= 2;
                    if (Math.abs(delta[idx]) > Math.abs(params[idx] / 1E6))
                        smallEnough = false;
                }
                if (smallEnough)
                    break;
            }
            step++;
        }
        return params;
    };
    ControllerOptimizationLocalVariation.prototype.Evaluate = function (params, stepHeight, stepTime, endTime) {
        var dt = endTime / this._NUM_TIME_STEPS;
        var t = 0;
        var w = 0;
        var err = 0;
        this._controlOptimization.OptimizeSetParameters(params, 0);
        this._controlLoop.Initialize(dt);
        while (t <= endTime) {
            if (t >= stepTime)
                w = stepHeight;
            var y = this._controlLoop.Calculate(w, dt);
            err += this._calcError.CalcError(w, y);
            t += dt;
        }
        return err;
    };
    return ControllerOptimizationLocalVariation;
}());
export { ControllerOptimizationLocalVariation };
//# sourceMappingURL=ControllerOptimizationLocalVariation.js.map