var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { ControlElement } from "./ControlElement";
import { Complex } from "../../Framework/Math/Complex";
var ControlPID = (function (_super) {
    __extends(ControlPID, _super);
    function ControlPID(Kp, Ki, Kd, uMin, uMax) {
        var _this = _super.call(this) || this;
        _this._e_n1 = 0;
        _this._e_sum = 0;
        _this._res = new Complex(0);
        _this._tmp = new Complex(0);
        _this._Kp = Kp;
        _this._Ki = Ki;
        _this._Kd = Kd;
        _this._uMin = uMin;
        _this._uMax = uMax;
        return _this;
    }
    ControlPID.prototype.GetType = function () {
        return "PID";
    };
    ControlPID.prototype.GetLowestTimeConstant = function () {
        var tn = -1;
        var tv = -1;
        if (this._Ki != 0) {
            tn = this._Kp / this._Ki;
        }
        if (this._Kp != 0) {
            tv = this._Kd / this._Kp;
        }
        if (tn == -1)
            return tv;
        if (tv == -1)
            return tn;
        return Math.min(tn, tv);
    };
    ControlPID.prototype.Initialize = function (_dt) {
        this._e_n1 = 0;
        this._e_sum = 0;
    };
    ControlPID.prototype.Calculate = function (e, dt) {
        var Kp = this._Kp;
        var Ki = this._Ki;
        var Kd = this._Kd;
        var eSumOld = this._e_sum;
        this._e_sum += e;
        var u = Kp * e + Ki * dt * this._e_sum + Kd * (e - this._e_n1) / dt;
        this._e_n1 = e;
        if (u > this._uMax) {
            u = this._uMax;
            this._e_sum = eSumOld;
        }
        if (u < this._uMin) {
            u = this._uMin;
            this._e_sum = eSumOld;
        }
        return u;
    };
    ControlPID.prototype.TimeFunctionStepResponse = function (t) {
        var n = 10;
        var d = n * Math.exp(-Math.PI * n * n * t * t);
        return this._Kp + this._Ki * t + this._Kd * d;
    };
    ControlPID.prototype.TransferFunction = function (s) {
        this._tmp.Set(this._Ki, 0);
        this._tmp.Div(s);
        this._res.Copy(s);
        this._res.MulR(this._Kd);
        this._res.Add(this._tmp);
        this._res.AddR(this._Kp);
        return this._res;
    };
    ControlPID.prototype.OptimizeGetNumParameters = function () {
        return 3;
    };
    ControlPID.prototype.OptimizeSetParameters = function (params, startIdx) {
        this._Kp = params[startIdx + 0];
        this._Ki = params[startIdx + 1];
        this._Kd = params[startIdx + 2];
        if (this._Kp < 0)
            this._Kp = 0;
        if (this._Ki < 0)
            this._Ki = 0;
        if (this._Kd < 0)
            this._Kd = 0;
    };
    return ControlPID;
}(ControlElement));
export { ControlPID };
//# sourceMappingURL=ControlPID.js.map