import { DifferentialEvolution } from "../../Framework/DifferentialEvolution/DifferentialEvolution";
import { ControlOptimizationError } from "./ControlOptimizationError";
var ControllerOptimizationDifferentialEvolution = (function () {
    function ControllerOptimizationDifferentialEvolution(controlLoop, controlOptimization) {
        this._NUM_TIME_STEPS = 5000;
        this._controlLoop = controlLoop;
        this._controlOptimization = controlOptimization;
    }
    ControllerOptimizationDifferentialEvolution.prototype.Optimize = function (limitMinParams, limitMaxParams, dither, maxNumSteps, stepHeight, stepTime, endTime, maxOvershot, weightingCtrlOut, weightingOvershot) {
        this._calcError = new ControlOptimizationError(this._controlLoop, stepHeight, maxOvershot, weightingCtrlOut, weightingOvershot);
        this._stepHeight = stepHeight;
        this._stepTime = stepTime;
        this._endTime = endTime;
        var numParams = this._controlOptimization.OptimizeGetNumParameters();
        var diffEvo = new DifferentialEvolution(this, numParams, dither);
        var step = 0;
        var initMinParamsArr = [];
        var initMaxParamsArr = [];
        var limitMinParamsArr = [];
        var limitMaxParamsArr = [];
        var idx = 0;
        for (idx = 0; idx < numParams; idx++) {
            initMinParamsArr[idx] = 0;
            initMaxParamsArr[idx] = 1;
            limitMinParamsArr[idx] = limitMinParams;
            limitMaxParamsArr[idx] = limitMaxParams;
        }
        diffEvo.CreateCandidates(initMinParamsArr, initMaxParamsArr, limitMinParamsArr, limitMaxParamsArr);
        for (step = 0; step < maxNumSteps; step++) {
            diffEvo.EvolutionStep();
        }
        return diffEvo.GetBestCandidate();
    };
    ControllerOptimizationDifferentialEvolution.prototype.RateCandidate = function (paramList) {
        return this.Evaluate(paramList, this._stepHeight, this._stepTime, this._endTime);
    };
    ControllerOptimizationDifferentialEvolution.prototype.Evaluate = function (params, stepHeight, stepTime, endTime) {
        var dt = endTime / this._NUM_TIME_STEPS;
        var t = 0;
        var w = 0;
        var err = 0;
        this._controlOptimization.OptimizeSetParameters(params, 0);
        this._controlLoop.Initialize(dt);
        while (t <= endTime) {
            if (t >= stepTime)
                w = stepHeight;
            var y = this._controlLoop.Calculate(w, dt);
            err += this._calcError.CalcError(w, y);
            t += dt;
        }
        return err;
    };
    return ControllerOptimizationDifferentialEvolution;
}());
export { ControllerOptimizationDifferentialEvolution };
//# sourceMappingURL=ControllerOptimizationDifferentialEvolution.js.map