var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { ControlElement } from "./ControlElement";
import { Complex } from "../../Framework/Math/Complex";
var ControlElementPT2 = (function (_super) {
    __extends(ControlElementPT2, _super);
    function ControlElementPT2(K, T, D) {
        var _this = _super.call(this) || this;
        _this._y_n1 = 0;
        _this._y_n2 = 0;
        _this._res = new Complex(0);
        if (T < 0)
            throw new Error("T must be greater then zero!");
        if (D < 0)
            throw new Error("D must be greater then zero!");
        _this._K = K;
        _this._T = T;
        _this._D = D;
        return _this;
    }
    ControlElementPT2.prototype.GetType = function () {
        return "PT2";
    };
    ControlElementPT2.prototype.GetLowestTimeConstant = function () {
        return this._T;
    };
    ControlElementPT2.prototype.Initialize = function (_dt) {
        this._y_n1 = 0;
        this._y_n2 = 0;
    };
    ControlElementPT2.prototype.Calculate = function (u, dt) {
        var K = this._K;
        var T = this._T;
        var D = this._D;
        var y_n2 = this._y_n2;
        var y_n1 = this._y_n1;
        var y = ((2 * y_n1 - y_n2) * T * T + 2 * dt * y_n1 * D * T + dt * dt * u * K) / (T * T + 2 * dt * D * T + dt * dt);
        this._y_n2 = this._y_n1;
        this._y_n1 = y;
        return y;
    };
    ControlElementPT2.prototype.TimeFunctionStepResponse = function (t) {
        var a = (this._D * this._D - 1) * this._T * this._T;
        var b = new Complex(a, 0);
        b.Sqrt();
        var c = new Complex();
        c.Copy(b);
        c.Neg();
        c.DivR(this._T * this._T);
        c.SubR(this._D / this._T);
        c.MulR(t);
        c.Exp();
        var d = new Complex();
        d.Copy(b);
        d.DivR(this._T * this._T);
        d.SubR(this._D / this._T);
        d.MulR(t);
        d.Exp();
        var e = new Complex();
        var f = new Complex();
        f.Copy(c);
        f.MulR(this._D * this._T);
        e.Sub(f);
        f.Copy(d);
        f.MulR(this._D * this._T);
        e.Add(f);
        f.Copy(c);
        f.Mul(b);
        e.Add(f);
        f.Copy(d);
        f.Mul(b);
        e.Add(f);
        e.Mul(b);
        e.MulR(-1 / (2 * a));
        e.AddR(1);
        e.MulR(this._K);
        return e.Real;
    };
    ControlElementPT2.prototype.TransferFunction = function (s) {
        this._res.Copy(s);
        this._res.MulR(this._T * this._T);
        this._res.AddR(2 * this._D * this._T);
        this._res.Mul(s);
        this._res.AddR(1);
        this._res.DivR(this._K);
        this._res.Inverse();
        return this._res;
    };
    ControlElementPT2.prototype.OptimizeGetNumParameters = function () {
        return 3;
    };
    ControlElementPT2.prototype.OptimizeSetParameters = function (params, startIdx) {
        this._K = params[startIdx + 0];
        this._T = params[startIdx + 1];
        this._D = params[startIdx + 2];
        if (this._T < 0)
            this._T = 0;
        if (this._D < 0)
            this._D = 0;
    };
    return ControlElementPT2;
}(ControlElement));
export { ControlElementPT2 };
//# sourceMappingURL=ControlElementPT2.js.map