var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { ControlElement } from "./ControlElement";
import { Complex } from "../../Framework/Math/Complex";
var ControlElementDeadTime = (function (_super) {
    __extends(ControlElementDeadTime, _super);
    function ControlElementDeadTime(T) {
        var _this = _super.call(this) || this;
        _this._y = 0;
        _this._res = new Complex(0);
        if (T < 0)
            throw new Error("T must be greater then zero!");
        _this._T = T;
        return _this;
    }
    ControlElementDeadTime.prototype.GetType = function () {
        return "DeadTime";
    };
    ControlElementDeadTime.prototype.GetLowestTimeConstant = function () {
        return this._T;
    };
    ControlElementDeadTime.prototype.Initialize = function (dt) {
        var cnt = Math.ceil((this._T / dt + 3) * 2);
        var idx;
        this._uArr = [];
        this._uArr.length = cnt;
        for (idx = 0; idx < cnt; idx++)
            this._uArr[idx] = 0;
        this._readIdx = 0;
        this._writeIdx = 0;
        this._tt = 0;
        this._y = 0;
    };
    ControlElementDeadTime.prototype.Calculate = function (u, dt) {
        this._uArr[this._writeIdx] = u;
        this._writeIdx++;
        if (this._writeIdx >= this._uArr.length)
            this._writeIdx = 0;
        if (this._tt < this._T) {
            this._tt += dt;
        }
        else {
            this._y = this._uArr[this._readIdx];
            this._readIdx++;
            if (this._readIdx >= this._uArr.length)
                this._readIdx = 0;
        }
        return this._y;
    };
    ControlElementDeadTime.prototype.TimeFunctionStepResponse = function (t) {
        return (t < this._T) ? 0 : 1;
    };
    ControlElementDeadTime.prototype.TransferFunction = function (s) {
        this._res.Copy(s);
        this._res.MulR(-this._T);
        this._res.Exp();
        return this._res;
    };
    ControlElementDeadTime.prototype.OptimizeGetNumParameters = function () {
        return 1;
    };
    ControlElementDeadTime.prototype.OptimizeSetParameters = function (params, startIdx) {
        this._T = params[startIdx + 0];
        if (this._T < 0)
            this._T = 0;
    };
    return ControlElementDeadTime;
}(ControlElement));
export { ControlElementDeadTime };
//# sourceMappingURL=ControlElementDeadTime.js.map