var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { ControlElement } from "./ControlElement";
import { Complex } from "../../Framework/Math/Complex";
var ControlElementPT1 = (function (_super) {
    __extends(ControlElementPT1, _super);
    function ControlElementPT1(K, T) {
        var _this = _super.call(this) || this;
        _this._y_n1 = 0;
        _this._res = new Complex(0);
        if (T < 0)
            throw new Error("T must be greater then zero!");
        _this._K = K;
        _this._T = T;
        return _this;
    }
    ControlElementPT1.prototype.GetType = function () {
        return "PT1";
    };
    ControlElementPT1.prototype.GetLowestTimeConstant = function () {
        return this._T;
    };
    ControlElementPT1.prototype.Initialize = function (_dt) {
        this._y_n1 = 0;
    };
    ControlElementPT1.prototype.Calculate = function (u, dt) {
        var K = this._K;
        var T = this._T;
        var y_n1 = this._y_n1;
        var y = (y_n1 * T + dt * u * K) / (T + dt);
        this._y_n1 = y;
        return y;
    };
    ControlElementPT1.prototype.TimeFunctionStepResponse = function (t) {
        return this._K * (1 - Math.exp(-t / this._T));
    };
    ControlElementPT1.prototype.TransferFunction = function (s) {
        this._res.Copy(s);
        this._res.MulR(this._T);
        this._res.AddR(1);
        this._res.DivR(this._K);
        this._res.Inverse();
        return this._res;
    };
    ControlElementPT1.prototype.OptimizeGetNumParameters = function () {
        return 2;
    };
    ControlElementPT1.prototype.OptimizeSetParameters = function (params, startIdx) {
        this._K = params[startIdx + 0];
        this._T = params[startIdx + 1];
        if (this._T < 0)
            this._T = 0;
    };
    return ControlElementPT1;
}(ControlElement));
export { ControlElementPT1 };
//# sourceMappingURL=ControlElementPT1.js.map