import { Complex } from "./Complex";
import { MathEx } from "./MathEx";
var LaplaceTransformInversionStehfest = (function () {
    function LaplaceTransformInversionStehfest() {
        this._N = 16;
        this._coeff = [];
        this._coeff.length = this._N;
        var n2 = MathEx.Trunc(this._N / 2);
        var sign = 1;
        var i;
        if ((n2 % 2) != 0)
            sign = -1;
        for (i = 0; i < this._N; i++) {
            var kMin = MathEx.Trunc((i + 2) / 2);
            var kMax = i + 1;
            var k = void 0;
            if (kMax > n2)
                kMax = n2;
            this._coeff[i] = 0;
            sign = -sign;
            for (k = kMin; k <= kMax; k++) {
                this._coeff[i] += (Math.pow(k, n2) / MathEx.Factorial(k))
                    * (MathEx.Factorial(2 * k) / MathEx.Factorial(2 * k - i - 1))
                    / MathEx.Factorial(n2 - k) / MathEx.Factorial(k - 1) / MathEx.Factorial(i + 1 - k);
            }
            this._coeff[i] = sign * this._coeff[i];
        }
    }
    LaplaceTransformInversionStehfest.prototype.GetName = function () { return "Stehfest"; };
    LaplaceTransformInversionStehfest.prototype.Calculate = function (F, tArr) {
        var yArr = [];
        var idx;
        var c = new Complex(0);
        for (idx = 0; idx < tArr.length; idx++) {
            var t = tArr[idx];
            if (t <= 0) {
                yArr[idx] = 0;
                continue;
            }
            var ln2t = Math.log(2.0) / t;
            var x = 0;
            var y = 0;
            var i = void 0;
            for (i = 0; i < this._coeff.length; i++) {
                x += ln2t;
                c.Set(x, 0);
                y += this._coeff[i] * F.Function(c).Real;
            }
            yArr[idx] = ln2t * y;
        }
        return yArr;
    };
    return LaplaceTransformInversionStehfest;
}());
export { LaplaceTransformInversionStehfest };
//# sourceMappingURL=LaplaceTransformInversionStehfest.js.map