import { DifferentialEvolution } from "../../Framework/DifferentialEvolution/DifferentialEvolution";
var ControlPathParamEstimation = (function () {
    function ControlPathParamEstimation(controlPath, measData) {
        this._controlPath = controlPath;
        this._measData = measData;
        var idx;
        this._dt = 0;
        for (idx = 1; idx < measData.length; idx++)
            this._dt += measData[idx][0] - measData[idx - 1][0];
        this._dt /= measData.length - 1;
    }
    ControlPathParamEstimation.prototype.Optimize = function (limitMinParams, limitMaxParams, dither, maxNumSteps, stepHeight, stepTime) {
        this._stepHeight = stepHeight;
        this._stepTime = stepTime;
        var numParams = this._controlPath.OptimizeGetNumParameters();
        var initMinParamsArr = [];
        var initMaxParamsArr = [];
        var limitMinParamsArr = [];
        var limitMaxParamsArr = [];
        var idx = 0;
        for (idx = 0; idx < numParams; idx++) {
            initMinParamsArr[idx] = 0;
            initMaxParamsArr[idx] = 1;
            limitMinParamsArr[idx] = limitMinParams;
            limitMaxParamsArr[idx] = limitMaxParams;
        }
        var diffEvo = new DifferentialEvolution(this, numParams, dither);
        var step = 0;
        diffEvo.CreateCandidates(initMinParamsArr, initMaxParamsArr, limitMinParamsArr, limitMaxParamsArr);
        for (step = 0; step < maxNumSteps; step++) {
            diffEvo.EvolutionStep();
        }
        return diffEvo.GetBestCandidate();
    };
    ControlPathParamEstimation.prototype.RateCandidate = function (paramList) {
        return this.Evaluate(paramList, this._stepHeight, this._stepTime);
    };
    ControlPathParamEstimation.prototype.Evaluate = function (params, stepHeight, stepTime) {
        var dt = this._dt;
        var t = 0;
        var w = 0;
        var err = 0;
        var idx;
        this._controlPath.OptimizeSetParameters(params, 0);
        this._controlPath.Initialize(dt);
        for (idx = 0; idx < this._measData.length; idx++) {
            if (t >= stepTime)
                w = stepHeight;
            var y = this._controlPath.Calculate(w, dt);
            var dy = y - this._measData[idx][1];
            err += dy * dy;
            t += dt;
        }
        return err;
    };
    return ControlPathParamEstimation;
}());
export { ControlPathParamEstimation };
//# sourceMappingURL=ControlPathParamEstimation.js.map