import { ControlLoop } from "./ControlLoop";
var ControlSimulation = (function () {
    function ControlSimulation(controlElement) {
        this._controlElement = controlElement;
    }
    ControlSimulation.prototype.StepResponseControlPath = function (stepHeight, stepTime, endTime) {
        var curveDt = endTime / 1000;
        var curveT = 0;
        var dt = this.GetDt(endTime);
        var t = 0;
        var u = 0;
        var curveU = [];
        var curveY = [];
        this._controlElement.Initialize(dt);
        while (t <= endTime) {
            if (t >= stepTime)
                u = stepHeight;
            var y = this._controlElement.Calculate(u, dt);
            curveT += dt;
            if (curveT >= curveDt) {
                curveU.push([t, u]);
                curveY.push([t, y]);
                curveT -= curveDt;
            }
            t += dt;
        }
        return [curveU, curveY];
    };
    ControlSimulation.prototype.StepResponseControlLoop = function (stepHeight, stepTime, endTime) {
        var curveDt = endTime / 1000;
        var curveT = 0;
        var dt = this.GetDt(endTime);
        var t = 0;
        var w = 0;
        var curveW = [];
        var curveU = [];
        var curveY = [];
        var maxY = 0;
        this._controlElement.Initialize(dt);
        while (t <= endTime) {
            if (t >= stepTime)
                w = stepHeight;
            var y = this._controlElement.Calculate(w, dt);
            if ((w >= 0) && (y > maxY))
                maxY = y;
            if ((w < 0) && (y < maxY))
                maxY = y;
            curveT += dt;
            if (curveT >= curveDt) {
                var u = 0;
                if (this._controlElement instanceof ControlLoop) {
                    var ctrlLoop = this._controlElement;
                    u = ctrlLoop.U;
                }
                curveW.push([t, w]);
                curveU.push([t, u]);
                curveY.push([t, y]);
                curveT -= curveDt;
            }
            t += dt;
        }
        return [curveW, curveU, curveY, maxY];
    };
    ControlSimulation.prototype.GetDt = function (endTime) {
        var MIN_NUM_POINTS = 100E3;
        var MAX_NUM_POINTS = 100E6;
        var lowT = this._controlElement.GetLowestTimeConstant();
        if (lowT == -1)
            return endTime / MIN_NUM_POINTS;
        var dt = lowT / 10000;
        var numPoints = endTime / dt;
        if (numPoints < MIN_NUM_POINTS)
            numPoints = MIN_NUM_POINTS;
        if (numPoints > MAX_NUM_POINTS)
            numPoints = MAX_NUM_POINTS;
        return endTime / numPoints;
    };
    return ControlSimulation;
}());
export { ControlSimulation };
//# sourceMappingURL=ControlSimulation.js.map