var Candidate = (function () {
    function Candidate(numParams) {
        this._paramList = [];
        this._paramList.length = numParams;
        this.Rating = 0;
    }
    Object.defineProperty(Candidate.prototype, "ParamList", {
        get: function () { return this._paramList; },
        enumerable: true,
        configurable: true
    });
    Candidate.prototype.Create = function (minParams, maxParams) {
        var idx;
        for (idx = 0; idx < this._paramList.length; idx++) {
            this._paramList[idx] = minParams[idx] + Math.random() * (maxParams[idx] - minParams[idx]);
        }
    };
    Candidate.prototype.MutateAndCrossover = function (candidateA, candidateB, candidateC, weightingFactor, crossoverProbability, minParams, maxParams) {
        var newCandidate = new Candidate(this._paramList.length);
        var randomIdx = Math.floor(Math.random() * this._paramList.length);
        var idx;
        for (idx = 0; idx < this._paramList.length; idx++) {
            var r = Math.random();
            if ((idx == randomIdx) || (r < crossoverProbability)) {
                var newParam = candidateC._paramList[idx] + weightingFactor * (candidateB._paramList[idx] - candidateA._paramList[idx]);
                if (newParam < minParams[idx])
                    newParam = minParams[idx];
                if (newParam > maxParams[idx])
                    newParam = maxParams[idx];
                newCandidate._paramList[idx] = newParam;
            }
            else {
                newCandidate._paramList[idx] = this._paramList[idx];
            }
        }
        return newCandidate;
    };
    return Candidate;
}());
var DifferentialEvolution = (function () {
    function DifferentialEvolution(iDifferentialEvolutionRating, numParams, dither) {
        this.NUM_CANDIDATES_FACTOR = 12;
        this.WEIGHTING_FACTOR_MIN = 0.5;
        this.WEIGHTING_FACTOR_MAX = 1.0;
        this.WEIGHTING_FACTOR_FIX = 0.8;
        this.CROSSOVER_PROBABILITY = 0.8;
        this._iDifferentialEvolutionRating = iDifferentialEvolutionRating;
        this._dither = (dither != null) ? dither : false;
        var numCandidates = numParams * this.NUM_CANDIDATES_FACTOR;
        this._candidateList = [];
        this._tempCandidateList = [];
        this._candidateList.length = numCandidates;
        this._tempCandidateList.length = numCandidates;
        var idx;
        for (idx = 0; idx < numCandidates; idx++) {
            this._candidateList[idx] = new Candidate(numParams);
        }
    }
    DifferentialEvolution.prototype.GetBestCandidate = function () {
        var bestCandidate = this._candidateList[0];
        var idx;
        for (idx = 1; idx < this._candidateList.length; idx++) {
            if (this._candidateList[idx].Rating < bestCandidate.Rating) {
                bestCandidate = this._candidateList[idx];
            }
        }
        return bestCandidate.ParamList;
    };
    DifferentialEvolution.prototype.CreateCandidates = function (initMinParams, initMaxParams, limitMinParams, limitMaxParams) {
        this._limitMinParams = limitMinParams;
        this._limitMaxParams = limitMaxParams;
        var idx;
        for (idx = 0; idx < this._candidateList.length; idx++) {
            this._candidateList[idx].Create(initMinParams, initMaxParams);
            this._candidateList[idx].Rating = this._iDifferentialEvolutionRating.RateCandidate(this._candidateList[idx].ParamList);
        }
    };
    DifferentialEvolution.prototype.EvolutionStep = function () {
        var idx;
        var candidateA;
        var candidateB;
        var candidateC;
        var newCandidate;
        var weightingFactor;
        if (this._dither)
            weightingFactor = this.WEIGHTING_FACTOR_MIN + Math.random() * (this.WEIGHTING_FACTOR_MAX - this.WEIGHTING_FACTOR_MIN);
        else
            weightingFactor = this.WEIGHTING_FACTOR_FIX;
        for (idx = 0; idx < this._candidateList.length; idx++) {
            _a = this.GetThreeRandomCandidates(idx), candidateA = _a[0], candidateB = _a[1], candidateC = _a[2];
            newCandidate = this._candidateList[idx].MutateAndCrossover(candidateA, candidateB, candidateC, weightingFactor, this.CROSSOVER_PROBABILITY, this._limitMinParams, this._limitMaxParams);
            newCandidate.Rating = this._iDifferentialEvolutionRating.RateCandidate(newCandidate.ParamList);
            if (newCandidate.Rating < this._candidateList[idx].Rating)
                this._tempCandidateList[idx] = newCandidate;
            else
                this._tempCandidateList[idx] = this._candidateList[idx];
        }
        for (idx = 0; idx < this._candidateList.length; idx++) {
            this._candidateList[idx] = this._tempCandidateList[idx];
        }
        var _a;
    };
    DifferentialEvolution.prototype.GetRandomCandidateIndex = function () {
        return Math.floor(Math.random() * this._candidateList.length);
    };
    DifferentialEvolution.prototype.GetThreeRandomCandidates = function (currentIdx) {
        var idxA;
        var idxB;
        var idxC;
        do {
            idxA = this.GetRandomCandidateIndex();
        } while (idxA == currentIdx);
        do {
            idxB = this.GetRandomCandidateIndex();
        } while ((idxB == currentIdx) || (idxB == idxA));
        do {
            idxC = this.GetRandomCandidateIndex();
        } while ((idxC == currentIdx) || (idxC == idxA) || (idxC == idxB));
        return [this._candidateList[idxA], this._candidateList[idxB], this._candidateList[idxC]];
    };
    return DifferentialEvolution;
}());
export { DifferentialEvolution };
//# sourceMappingURL=DifferentialEvolution.js.map