import { Complex } from "./Complex";
import { MathEx } from "./MathEx";
var LaplaceTransformInversionTalbot = (function () {
    function LaplaceTransformInversionTalbot(n, shift) {
        if (n === void 0) { n = 128; }
        if (shift === void 0) { shift = 0.0; }
        this._n = n;
        this._shift = shift;
    }
    LaplaceTransformInversionTalbot.prototype.GetName = function () { return "Zakian"; };
    LaplaceTransformInversionTalbot.prototype.Calculate = function (F, tArr) {
        var yArr = [];
        var idx;
        var c1 = 0.5017;
        var c2 = 0.6407;
        var c3 = 0.6122;
        var c4 = new Complex(0, 0.2645);
        var z = new Complex(0);
        var dz = new Complex(0);
        var ans = new Complex(0);
        var e = new Complex(0);
        var r = new Complex(0);
        for (idx = 0; idx < tArr.length; idx++) {
            var t = tArr[idx];
            if (t <= 0) {
                yArr[idx] = 0;
                continue;
            }
            var h = 2 * Math.PI / this._n;
            ans.Set(0);
            var k = void 0;
            for (k = 0; k <= this._n; k++) {
                var theta = -Math.PI + (k + 0.5) * h;
                var a = this._shift + this._n / t * (c1 * theta / Math.tan(c2 * theta) - c3);
                var b = this._n / t * (-c1 * c2 * theta / MathEx.Sqr(Math.sin(c2 * theta)) + c1 / Math.tan(c2 * theta));
                z.Copy(c4);
                z.MulR(this._n / t * theta);
                z.AddR(a);
                dz.Copy(c4);
                dz.MulR(this._n / t);
                dz.AddR(b);
                e.Copy(z);
                e.MulR(t);
                e.Exp();
                e.Mul(dz);
                e.Mul(F.Function(z));
                ans.Add(e);
            }
            r.Set(0, 2 * Math.PI);
            r.DivR(h);
            r.Inverse();
            r.Mul(ans);
            yArr[idx] = r.Real;
        }
        return yArr;
    };
    return LaplaceTransformInversionTalbot;
}());
export { LaplaceTransformInversionTalbot };
//# sourceMappingURL=LaplaceTransformInversionTalbot.js.map