/*
    Copyright (C) 2017  Torsten Brischalle
    email: torsten@brischalle.de
    web: http://www.aaabbb.de
    
    MIT License

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
*/

import { Gui } from "Framework/Gui/Gui";

import { ControlAppPathStepResponse } from "../../ControlAppPathStepResponse/src/ControlAppPathStepResponse";
import { ControlTools } from "../../shared/ControlTools";
import { ControlPath } from "../../shared/ControlPath";
import { ControlPathParamEstimation } from "../../shared/ControlPathParamEstimation";

//-------------------------------------------------------------------------------------------------

export class ControlAppPathParamEstimation extends ControlAppPathStepResponse
{
    public static Inst: ControlAppPathParamEstimation = new ControlAppPathParamEstimation();

    //-------------------------------------------------------------------------------------------------

    public Init(): void
    {
        this.ControlAppInit();
    }

    //-------------------------------------------------------------------------------------------------

    public ControlPathEstimateParamsDiffEvo(): void
    {
        try
        {
            let maxNumSteps: number = Gui.GetNumber("MAX_NUM_STEPS");
            let uArr: [number, number][];
            let yArr: [number, number][];

            [uArr, yArr] = ControlTools.ParseMeasDataString(document.getElementById("MEAS_DATA").textContent);

            let stepHeight: number;
            let stepTime: number;
            let endTime: number;

            [stepHeight, stepTime, endTime] = ControlTools.CalcStepParams(uArr);

            Gui.Set("STEP_HEIGHT", stepHeight);
            Gui.Set("STEP_TIME", stepTime);
            Gui.Set("END_TIME", endTime);

            let ctrlPath: ControlPath = this.CreateControlPathFromGui();

            // Parameter der Regelstrecke so optimieren, dass sie möglichst gut mit den
            // gemessenen Werten überein stimmen
            let ctrlOpti: ControlPathParamEstimation = new ControlPathParamEstimation(ctrlPath, yArr);

            let minParams: number = 0;
            let maxParams: number = 1000;
            let dither: boolean = true;

            let params: number[] = ctrlOpti.Optimize(minParams, maxParams, dither, maxNumSteps, stepHeight, stepTime);

            // Parameter anzeigen in der HTML Seite
            this.ShowControlParamsInGui(params);

            Gui.Set("STEP_HEIGHT", stepHeight);
            Gui.Set("STEP_TIME", stepTime);
            Gui.Set("END_TIME", endTime);

            this.ControlPathStepResponse(yArr);
        }
        catch (err)
        {
            alert(err.message);
        }
    }

    //-------------------------------------------------------------------------------------------------

    public ControlPathEstimateParamsStepResponse(): void
    {
        try
        {
            let uArr: [number, number][];
            let yArr: [number, number][];

            [uArr, yArr] = ControlTools.ParseMeasDataString(document.getElementById("MEAS_DATA").textContent);

            this.ControlPathStepResponse(yArr);
        }
        catch (err)
        {
            alert(err.message);
        }
    }

    //-------------------------------------------------------------------------------------------------
}

//-------------------------------------------------------------------------------------------------
