var Complex = (function () {
    function Complex(re, im) {
        this._re = 0;
        this._im = 0;
        this.Set(re, im);
    }
    Object.defineProperty(Complex.prototype, "Real", {
        get: function () { return this._re; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Complex.prototype, "Imag", {
        get: function () { return this._im; },
        enumerable: true,
        configurable: true
    });
    Complex.Create = function (a, b) {
        var c = new Complex();
        c.Parse(a, b);
        return c;
    };
    Complex.prototype.Set = function (re, im) {
        this._re = (re != null) ? re : 0;
        this._im = (im != null) ? im : 0;
    };
    Complex.prototype.Copy = function (src) {
        this._re = src._re;
        this._im = src._im;
    };
    Complex.prototype.clone = function () {
        return new Complex(this._re, this._im);
    };
    Complex.prototype.Sign = function () {
        var abs = this.Abs();
        this._re /= abs;
        this._im /= abs;
    };
    Complex.prototype.Add = function (a) {
        this._re += a._re;
        this._im += a._im;
    };
    Complex.prototype.AddR = function (a) {
        this._re += a;
    };
    Complex.prototype.Sub = function (a) {
        this._re -= a._re;
        this._im -= a._im;
    };
    Complex.prototype.SubR = function (a) {
        this._re -= a;
    };
    Complex.prototype.Mul = function (a) {
        if ((a._im === 0) && (this._im === 0)) {
            this._re *= a._re;
            this._im = 0;
        }
        else {
            var re = this._re;
            this._re = this._re * a._re - this._im * a._im;
            this._im = re * a._im + this._im * a._re;
        }
    };
    Complex.prototype.MulR = function (a) {
        this._re *= a;
        this._im *= a;
    };
    Complex.prototype.Div = function (a) {
        var re = this._re;
        var im = this._im;
        if (a._im === 0) {
            if (a._re === 0) {
                this._re = (re !== 0) ? (re / 0) : 0;
                this._im = (im !== 0) ? (im / 0) : 0;
            }
            else {
                this._re /= a._re;
                this._im /= a._im;
            }
        }
        if (Math.abs(a._re) < Math.abs(a._im)) {
            var x = a._re / a._im;
            var t = a._re * x + a._im;
            this._re = (re * x + im) / t;
            this._im = (im * x - re) / t;
        }
        else {
            var x = a._im / a._re;
            var t = a._im * x + a._re;
            this._re = (re + im * x) / t;
            this._im = (im - re * x) / t;
        }
    };
    Complex.prototype.DivR = function (a) {
        this._re /= a;
        this._im /= a;
    };
    Complex.prototype.Pow = function (a) {
        var re = this._re;
        var im = this._im;
        if ((re === 0) && (im === 0)) {
            this._re = 0;
            this._im = 0;
            return;
        }
        var arg = Math.atan2(im, re);
        var loh = Complex.LogHypot(re, im);
        if (a._im === 0) {
            if ((im === 0) && (re >= 0)) {
                this._re = Math.pow(re, a._re);
                this._im = 0;
                return;
            }
            else if (re === 0) {
                switch (a._re % 4) {
                    case 0:
                        this._re = Math.pow(im, a._re);
                        this._im = 0;
                        return;
                    case 1:
                        this._re = 0;
                        this._im = Math.pow(im, a._re);
                        return;
                    case 2:
                        this._re = -Math.pow(im, a._re);
                        this._im = 0;
                        return;
                    case 3:
                        this._re = 0;
                        this._im = -Math.pow(im, a._re);
                        return;
                }
            }
        }
        var x = Math.exp(a._re * loh - a._im * arg);
        var y = a._im * loh + a._re * arg;
        this._re = x * Math.cos(y);
        this._im = x * Math.sin(y);
    };
    Complex.prototype.Sqrt = function () {
        var re = this._re;
        var im = this._im;
        var r = this.Abs();
        var rr;
        var ii;
        if (re >= 0) {
            if (im === 0) {
                this._re = Math.sqrt(re);
                this._im = 0;
                return;
            }
            rr = 0.5 * Math.sqrt(2.0 * (r + re));
        }
        else {
            rr = Math.abs(im) / Math.sqrt(2 * (r - re));
        }
        if (re <= 0)
            ii = 0.5 * Math.sqrt(2.0 * (r - re));
        else
            ii = Math.abs(im) / Math.sqrt(2 * (r + re));
        this._re = rr;
        this._im = (im < 0) ? -ii : ii;
    };
    Complex.prototype.Exp = function () {
        var a = Math.exp(this._re);
        this._re = a * Math.cos(this._im);
        this._im = a * Math.sin(this._im);
    };
    Complex.prototype.Log = function () {
        var re = this._re;
        var im = this._im;
        this._re = Complex.LogHypot(re, im);
        this._im = Math.atan2(im, re);
    };
    Complex.prototype.Abs = function () {
        return Complex.Hypot(this._re, this._im);
    };
    Complex.prototype.Arg = function () {
        return Math.atan2(this._im, this._re);
    };
    Complex.prototype.Sin = function () {
        var re = this._re;
        var im = this._im;
        this._re = Math.sin(re) * Complex.CoshR(im);
        this._im = Math.cos(re) * Complex.SinhR(im);
    };
    Complex.prototype.Cos = function () {
        var re = this._re;
        var im = this._im;
        this._re = Math.cos(re) * Complex.CoshR(im);
        this._im = -Math.sin(re) * Complex.SinhR(im);
    };
    Complex.prototype.Tan = function () {
        var a = 2 * this._re;
        var b = 2 * this._im;
        var d = Math.cos(a) + Complex.CoshR(b);
        this._re = Math.sin(a) / d;
        this._im = Complex.SinhR(b) / d;
    };
    Complex.prototype.Cot = function () {
        var a = 2 * this._re;
        var b = 2 * this._im;
        var d = Math.cos(a) - Complex.CoshR(b);
        this._re = -Math.sin(a) / d;
        this._im = Complex.SinhR(b) / d;
    };
    Complex.prototype.Sec = function () {
        var a = this._re;
        var b = this._im;
        var d = 0.5 * Complex.CoshR(2 * b) + 0.5 * Math.cos(2 * a);
        this._re = Math.cos(a) * Complex.CoshR(b) / d;
        this._im = Math.sin(a) * Complex.SinhR(b) / d;
    };
    Complex.prototype.Csc = function () {
        var a = this._re;
        var b = this._im;
        var d = 0.5 * Complex.CoshR(2 * b) - 0.5 * Math.cos(2 * a);
        this._re = Math.sin(a) * Complex.CoshR(b) / d;
        this._im = -Math.cos(a) * Complex.SinhR(b) / d;
    };
    Complex.prototype.Asin = function () {
        var a = this._re;
        var b = this._im;
        var t1 = new Complex(b * b - a * a + 1, -2 * a * b);
        t1.Sqrt();
        var t2 = new Complex(t1.Real - b, t1.Imag + a);
        t2.Log();
        this._re = t2.Imag;
        this._im = -t2.Real;
    };
    Complex.prototype.Acos = function () {
        var a = this._re;
        var b = this._im;
        var t1 = new Complex(b * b - a * a + 1, -2 * a * b);
        t1.Sqrt();
        var t2 = new Complex(t1.Real - b, t1.Imag + a);
        t2.Log();
        this._re = Math.PI / 2 - t2.Imag;
        this._im = t2.Real;
    };
    Complex.prototype.Atan = function () {
        var a = this._re;
        var b = this._im;
        if (a === 0) {
            if (b === 1) {
                this._re = 0;
                this._im = Infinity;
                return;
            }
            if (b === -1) {
                this._re = 0;
                this._im = -Infinity;
                return;
            }
        }
        var d = a * a + (1.0 - b) * (1.0 - b);
        var t1 = new Complex((1 - b * b - a * a) / d, -2 * a / d);
        t1.Log();
        this._re = -0.5 * t1.Imag;
        this._im = 0.5 * t1.Real;
    };
    Complex.prototype.Acot = function () {
        var a = this._re;
        var b = this._im;
        if (b === 0) {
            this._re = Math.atan2(1, a);
            this._im = 0;
            return;
        }
        var d = a * a + b * b;
        var t;
        if (d !== 0)
            t = new Complex(a / d, -b / d);
        else
            t = new Complex((a !== 0) ? a / 0 : 0, (b !== 0) ? -b / 0 : 0);
        t.Atan();
        this.Copy(t);
    };
    Complex.prototype.Asec = function () {
        var a = this._re;
        var b = this._im;
        if ((a === 0) && (b === 0)) {
            this._re = 0;
            this._im = Infinity;
            return;
        }
        var d = a * a + b * b;
        var t;
        if (d !== 0)
            t = new Complex(a / d, -b / d);
        else
            t = new Complex((a !== 0) ? a / 0 : 0, (b !== 0) ? -b / 0 : 0);
        t.Acos();
        this.Copy(t);
    };
    Complex.prototype.Acsc = function () {
        var a = this._re;
        var b = this._im;
        if ((a === 0) && (b === 0)) {
            this._re = Math.PI / 2;
            this._im = Infinity;
            return;
        }
        var d = a * a + b * b;
        var t;
        if (d !== 0)
            t = new Complex(a / d, -b / d);
        else
            t = new Complex((a !== 0) ? a / 0 : 0, (b !== 0) ? -b / 0 : 0);
        t.Asin();
        this.Copy(t);
    };
    Complex.prototype.Sinh = function () {
        var a = this._re;
        var b = this._im;
        this._re = Complex.SinhR(a) * Math.cos(b);
        this._im = Complex.CoshR(a) * Math.sin(b);
    };
    Complex.prototype.Cosh = function () {
        var a = this._re;
        var b = this._im;
        this._re = Complex.CoshR(a) * Math.cos(b);
        this._im = Complex.SinhR(a) * Math.sin(b);
    };
    Complex.prototype.Tanh = function () {
        var a = 2 * this._re;
        var b = 2 * this._im;
        var d = Complex.CoshR(a) + Math.cos(b);
        this._re = Complex.SinhR(a) / d;
        this._im = Math.sin(b) / d;
    };
    Complex.prototype.Coth = function () {
        var a = 2 * this._re;
        var b = 2 * this._im;
        var d = Complex.CoshR(a) - Math.cos(b);
        this._re = Complex.SinhR(a) / d;
        this._im = -Math.sin(b) / d;
    };
    Complex.prototype.Csch = function () {
        var a = this._re;
        var b = this._im;
        var d = Math.cos(2 * b) - Complex.CoshR(2 * a);
        this._re = -2 * Complex.SinhR(a) * Math.cos(b) / d;
        this._im = 2 * Complex.CoshR(a) * Math.sin(b) / d;
    };
    Complex.prototype.Sech = function () {
        var a = this._re;
        var b = this._im;
        var d = Math.cos(2 * b) + Complex.CoshR(2 * a);
        this._re = 2 * Complex.CoshR(a) * Math.cos(b) / d;
        this._im = -2 * Complex.SinhR(a) * Math.sin(b) / d;
    };
    Complex.prototype.Asinh = function () {
        var t = new Complex(this._im, -this._re);
        t.Asin();
        this._re = -t.Imag;
        this._im = t.Real;
    };
    Complex.prototype.Acosh = function () {
        var t = new Complex();
        t.Copy(this);
        t.Acos();
        if (t.Imag <= 0) {
            this._re = -t.Imag;
            this._im = t.Real;
        }
        else {
            this._re = t.Imag;
            this._im = -t.Real;
        }
    };
    Complex.prototype.Atanh = function () {
        var a = this._re;
        var b = this._im;
        var noIM = (a > 1) && (b === 0);
        var oneMinus = 1 - a;
        var onePlus = 1 + a;
        var d = oneMinus * oneMinus + b * b;
        var x;
        if (d !== 0)
            x = new Complex((onePlus * oneMinus - b * b) / d, (b * oneMinus + onePlus * b) / d);
        else
            x = new Complex((a !== -1) ? (a / 0) : 0, (b !== 0) ? (b / 0) : 0);
        this._re = Complex.LogHypot(x.Real, x.Imag) / 2;
        this._im = Math.atan2(x.Imag, x.Real) / 2;
        if (noIM)
            this._im = -this._im;
    };
    Complex.prototype.Acoth = function () {
        var a = this._re;
        var b = this._im;
        if ((a === 0) && (b === 0)) {
            this._re = 0;
            this._im = Math.PI / 2;
            return;
        }
        var d = a * a + b * b;
        var t;
        if (d !== 0)
            t = new Complex(a / d, -b / d);
        else
            t = new Complex((a !== 0) ? a / 0 : 0, (b !== 0) ? -b / 0 : 0);
        t.Atanh();
        this.Copy(t);
    };
    Complex.prototype.Acsch = function () {
        var a = this._re;
        var b = this._im;
        if (b === 0) {
            this._re = (a !== 0) ? Math.log(a + Math.sqrt(a * a + 1)) : Infinity;
            this._im = 0;
            return;
        }
        var d = a * a + b * b;
        var t;
        if (d !== 0)
            t = new Complex(a / d, -b / d);
        else
            t = new Complex((a !== 0) ? a / 0 : 0, (b !== 0) ? -b / 0 : 0);
        t.Asinh();
        this.Copy(t);
    };
    Complex.prototype.Asech = function () {
        var a = this._re;
        var b = this._im;
        if ((a === 0) && (b === 0)) {
            this._re = Infinity;
            this._im = 0;
            return;
        }
        var d = a * a + b * b;
        var t;
        if (d !== 0)
            t = new Complex(a / d, -b / d);
        else
            t = new Complex((a !== 0) ? a / 0 : 0, (b !== 0) ? -b / 0 : 0);
        t.Acosh();
        this.Copy(t);
    };
    Complex.prototype.Inverse = function () {
        var a = this._re;
        var b = this._im;
        var d = a * a + b * b;
        this._re = a !== 0 ? a / d : 0;
        this._im = b !== 0 ? -b / d : 0;
    };
    Complex.prototype.Conjugate = function () {
        this._im = -this._im;
    };
    Complex.prototype.Neg = function () {
        this._re = -this._re;
        this._im = -this._im;
    };
    Complex.prototype.Ceil = function (places) {
        places = Math.pow(10, places);
        this._re = Math.ceil(this._re * places) / places;
        this._im = Math.ceil(this._im * places) / places;
    };
    Complex.prototype.Floor = function (places) {
        places = Math.pow(10, places);
        this._re = Math.floor(this._re * places) / places;
        this._im = Math.floor(this._im * places) / places;
    };
    Complex.prototype.Round = function (places) {
        places = Math.pow(10, places);
        this._re = Math.round(this._re * places) / places;
        this._im = Math.round(this._im * places) / places;
    };
    Complex.prototype.Equals = function (a) {
        return (Math.abs(a._re - this._re) <= Complex.EPSILON) &&
            (Math.abs(a._im - this._im) <= Complex.EPSILON);
    };
    Complex.prototype.toString = function () {
        var a = this._re;
        var b = this._im;
        var ret = "";
        if (isNaN(a) || isNaN(b))
            return 'NaN';
        if (a !== 0)
            ret += a;
        if (b !== 0) {
            if (a !== 0)
                ret += (b < 0) ? " - " : " + ";
            else if (b < 0)
                ret += "-";
            b = Math.abs(b);
            if (1 !== b)
                ret += b;
            ret += "i";
        }
        if (ret.length === 0)
            return "0";
        return ret;
    };
    Complex.prototype.toVector = function () {
        return [this._re, this._im];
    };
    Complex.prototype.valueOf = function () {
        if (this._im === 0)
            return this._re;
        return null;
    };
    Complex.prototype.isNaN = function () {
        return isNaN(this._re) || isNaN(this._im);
    };
    Complex.CoshR = function (x) {
        return (Math.exp(x) + Math.exp(-x)) * 0.5;
    };
    Complex.SinhR = function (x) {
        return (Math.exp(x) - Math.exp(-x)) * 0.5;
    };
    Complex.Hypot = function (x, y) {
        var a = Math.abs(x);
        var b = Math.abs(y);
        if (a < 3000 && b < 3000) {
            return Math.sqrt(a * a + b * b);
        }
        if (a < b) {
            a = b;
            b = x / y;
        }
        else {
            b = y / x;
        }
        return a * Math.sqrt(1 + b * b);
    };
    Complex.LogHypot = function (a, b) {
        var _a = Math.abs(a);
        var _b = Math.abs(b);
        if (a === 0)
            return Math.log(_b);
        if (b === 0)
            return Math.log(_a);
        if ((_a < 3000) && (_b < 3000))
            return Math.log(a * a + b * b) * 0.5;
        return Math.log(a / Math.cos(Math.atan2(b, a)));
    };
    Complex.prototype.Parse = function (a, b) {
        this._re = 0;
        this._im = 0;
        if (a instanceof Complex) {
            this.Copy(a);
            return;
        }
        switch (typeof a) {
            case "string":
                this.ParseString(a);
                break;
            case "number":
                this._re = a;
                if (typeof b === "number")
                    this._im = b;
                break;
            case "object":
                if (("re" in a) && ("im" in a) && (typeof a.re === "number") && (typeof a.im === "number")) {
                    this._re = a["re"];
                    this._im = a["im"];
                }
                else if (("abs" in a) && ("arg" in a) && (typeof a["abs"] === "number") && (typeof a["arg"] === "number")) {
                    this._re = a["abs"] * Math.cos(a["arg"]);
                    this._im = a["abs"] * Math.sin(a["arg"]);
                }
                else if (("r" in a) && ("phi" in a) && (typeof a["r"] === "number") && (typeof a["phi"] === "number")) {
                    this._re = a["r"] * Math.cos(a["phi"]);
                    this._im = a["r"] * Math.sin(a["phi"]);
                }
                else
                    throw new SyntaxError("invalid parameter");
                break;
            default:
                throw new SyntaxError("invalid parameter");
        }
    };
    Complex.prototype.ParseString = function (s) {
        var tokens = s.match(/\d+\.?\d*e[+-]?\d+|\d+\.?\d*|\.\d+|./g);
        var plus = 1;
        var minus = 0;
        var i;
        if (tokens === null)
            throw new SyntaxError("invalid parameter");
        for (i = 0; i < tokens.length; i++) {
            var c = tokens[i];
            if (c === ' ' || c === '\t' || c === '\n') {
            }
            else if (c === '+')
                plus++;
            else if (c === '-')
                minus++;
            else if ((c === 'i') || (c === 'I')) {
                if (plus + minus === 0) {
                    throw new SyntaxError("invalid parameter");
                }
                if (tokens[i + 1] !== ' ' && !isNaN(Number(tokens[i + 1]))) {
                    this._im += parseFloat((minus % 2 ? '-' : '') + tokens[i + 1]);
                    i++;
                }
                else {
                    this._im += parseFloat((minus % 2 ? '-' : '') + '1');
                }
                plus = minus = 0;
            }
            else {
                if (plus + minus === 0 || isNaN(Number(c)))
                    throw new SyntaxError("invalid parameter");
                if (tokens[i + 1] === 'i' || tokens[i + 1] === 'I') {
                    this._im += parseFloat((minus % 2 ? '-' : '') + c);
                    i++;
                }
                else
                    this._re += parseFloat((minus % 2 ? '-' : '') + c);
                plus = minus = 0;
            }
        }
        if (plus + minus > 0) {
            throw new SyntaxError("invalid parameter");
        }
    };
    Complex.ZERO = new Complex(0, 0);
    Complex.ONE = new Complex(1, 0);
    Complex.I = new Complex(0, 1);
    Complex.PI = new Complex(Math.PI, 0);
    Complex.E = new Complex(Math.E, 0);
    Complex.EPSILON = 1e-16;
    return Complex;
}());
export { Complex };
//# sourceMappingURL=Complex.js.map