/*
    Copyright (C) 2017  Torsten Brischalle
    email: torsten@brischalle.de
    web: http://www.aaabbb.de
    
    MIT License

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
*/

import { Gui } from "Framework/Gui/Gui";
import { Graph } from "Framework/Gui/Graph";
import { Derivative } from "Framework/Math/Derivative";

import { ControlApp } from "../../shared/ControlApp";
import { ControlPath } from "../../shared/ControlPath";
import { ControlSimulation } from "../../shared/ControlSimulation";

//-------------------------------------------------------------------------------------------------

export class ControlAppPathStepResponse extends ControlApp
{
    public static Inst: ControlAppPathStepResponse = new ControlAppPathStepResponse();

    //-------------------------------------------------------------------------------------------------

    public Init(): void
    {
        this.ControlAppInit();
    }

    //-------------------------------------------------------------------------------------------------

    public ControlPathStepResponse(measuredData?: [number, number][]): void
    {
        try
        {
            let stepHeight: number = Gui.GetNumber("STEP_HEIGHT");
            let stepTime: number = Gui.GetNumber("STEP_TIME");
            let endTime: number = Gui.GetNumber("END_TIME");
            let show1stDerivative: boolean = Gui.IsCheckedWithDefault("SHOW_DERIVATIVE_1", false);
            let show2ndDerivative: boolean = Gui.IsCheckedWithDefault("SHOW_DERIVATIVE_2", false);

            let ctrlPath: ControlPath = this.CreateControlPathFromGui();
            let ctrlSim: ControlSimulation = new ControlSimulation(ctrlPath);
            let curveU: [number, number][];
            let curveY: [number, number][];

            [curveU, curveY] = ctrlSim.StepResponseControlPath(stepHeight, stepTime, endTime);

            let curveNameList: string[] = ["u(t)", "y(t)"];
            let yAxisList: number[] = [1, 1];
            let curveColorList: string[] = ["#0000FF", "#FF0000"];
            let curveDataList: [number, number][][] = [curveU, curveY];

            if (measuredData)
            {
                curveNameList.push("measured(t)");
                yAxisList.push(1);
                curveColorList.push("#00FF00");
                curveDataList.push(measuredData);
            }

            if (show1stDerivative)
            {
                let curveDy1: [number, number][] = Derivative.DerivativeEx(curveY);

                curveNameList.push("1st derivative y(t)");
                yAxisList.push(1);
                curveColorList.push("#9400D3");
                curveDataList.push(curveDy1);
            }

            if (show2ndDerivative)
            {
                let curveDy1: [number, number][] = Derivative.DerivativeEx(curveY);
                let curveDy2: [number, number][] = Derivative.DerivativeEx(curveDy1);

                curveNameList.push("2nd derivative y(t)");
                yAxisList.push(1);
                curveColorList.push("#4B0082");
                curveDataList.push(curveDy2);
            }

            Graph.Draw("DIV_GRAPH", "Step response", "Time", "Amplitude", "Amplitude", curveNameList, yAxisList, curveColorList, curveDataList);
        }
        catch (err)
        {
            alert(err.message);
        }
    }

    //-------------------------------------------------------------------------------------------------
}

//-------------------------------------------------------------------------------------------------
