
/*
    Copyright (C) 2017  Torsten Brischalle
    email: torsten@brischalle.de
    web: http://www.aaabbb.de
    
    MIT License

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
*/

import { Gui } from "../../Framework/Gui/Gui";

import { GuiControlElement } from "./GuiControlElement";
import { ControlPath } from "./ControlPath";

//-------------------------------------------------------------------------------------------------

export abstract class ControlApp
{
    //-------------------------------------------------------------------------------------------------

    public readonly MAX_NUM_CONTROL_ELEMENTS: number = 4;

    //-------------------------------------------------------------------------------------------------

    private _controlElementList: GuiControlElement[];
    private _controlPathImg: HTMLObjectElement;

    //-------------------------------------------------------------------------------------------------

    protected ControlAppInit(): void
    {
        let idx: number;

        this._controlPathImg = <HTMLObjectElement>document.getElementById("PIC_CTRL");
        console.assert(this._controlPathImg != null, "PIC_CTRL is null");

        this._controlElementList = [];

        for (idx = 1; idx <= this.MAX_NUM_CONTROL_ELEMENTS; idx++)
            this._controlElementList[idx] = GuiControlElement.Create("NONE", idx, this._controlPathImg);

        for (idx = 1; idx <= this.MAX_NUM_CONTROL_ELEMENTS; idx++)
            this.CtrlElemSelected(idx);
    }

    //-------------------------------------------------------------------------------------------------

    protected CreateControlPathFromGui(): ControlPath
    {
        let ctrlPath: ControlPath = new ControlPath();
        let idx: number;

        for (idx = 1; idx <= this.MAX_NUM_CONTROL_ELEMENTS; idx++)
        {
            if (this._controlElementList[idx].CtrlType != "NONE")
                ctrlPath.Add(this._controlElementList[idx].CreateControl());
        }

        return ctrlPath;
    }

    //-------------------------------------------------------------------------------------------------

    protected ShowControlParamsInGui(ctrlParams: number[]): void
    {
        // Parameter anzeigen in der HTML Seite
        let idx: number;
        let paramIdx: number = 0;

        for (idx = 1; idx <= this.MAX_NUM_CONTROL_ELEMENTS; idx++)
        {
            if (this._controlElementList[idx].CtrlType != "NONE")
                this._controlElementList[idx].ShowParamVals(ctrlParams, paramIdx);

            paramIdx += this._controlElementList[idx].GetNumParams();
        }
    }

    //-------------------------------------------------------------------------------------------------

    protected SetControlParamInGuiPic(paramName: string, valStr: string): void
    {
        this._controlPathImg.contentDocument.getElementById(paramName).textContent = valStr;
    }


    //-------------------------------------------------------------------------------------------------

    public CtrlElemSelected(idx: number)
    {
        let ctrlType: string = Gui.GetString("CTRL_ELEM_TYPE_" + idx);
        let ctrl: GuiControlElement = GuiControlElement.Create(ctrlType, idx, this._controlPathImg);

        this._controlElementList[idx] = ctrl;

        ctrl.Show();
        ctrl.ShowParams();
    }

    //-------------------------------------------------------------------------------------------------
}

//-------------------------------------------------------------------------------------------------
