
/*
    Copyright (C) 2016  Torsten Brischalle
    email: torsten@brischalle.de
    web: http://www.aaabbb.de
    
    MIT License

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
*/

import { ControlElement } from "./ControlElement";
import { Complex } from "../../Framework/Math/Complex";

//-------------------------------------------------------------------------------------------------

export class ControlComparator extends ControlElement
{
    private _Threshold: number;

    private _uMin: number;
    private _uMax: number;

    private _time: number;
    private _direction: number;
    private _lastTime: number;
    private _period: number;
    private _periodCnt: number;

    public get Period(): number { return this._period / this._periodCnt; }
    public get Amplitude(): number { return (this._uMax - this._uMin) / 2; }

    public constructor(threshold: number, uMin: number, uMax: number)
    {
        super();

        this._Threshold = threshold;

        this._uMin = uMin;
        this._uMax = uMax;
    }

    public GetType(): string
    {
        return "Comparator";
    }

    public GetLowestTimeConstant(): number
    {
        return -1;
    }

    public Initialize(_dt: number): void
    {
        this._time = 0;
        this._direction = 0;
        this._lastTime = -1;
        this._period = 0;
        this._periodCnt = 0;
    }

    public Calculate(e: number, dt: number): number
    {
        let u: number;

        this._time += dt;

        if (e + this._Threshold > 0)
        {
            if (this._direction == -1)
            {
                if (this._lastTime >= 0)
                {
                    this._period += this._time - this._lastTime;
                    this._periodCnt++;
                }

                this._lastTime = this._time;
            }

            u = this._uMax;
            this._direction = 1;
        }
        else
        {
            u = this._uMin;
            this._direction = -1;
        }

        return u;
    }

    public TimeFunctionStepResponse(_t: number): number
    {
        return 0;
    }

    private _res: Complex = new Complex(0);

    public TransferFunction(_s: Complex): Complex
    {
        this._res.Set(0, 0);
        return this._res;
    }

    public OptimizeGetNumParameters(): number
    {
        return 3;
    }

    public OptimizeSetParameters(params: number[], startIdx: number): void
    {
        this._Threshold = params[startIdx + 0];

        this._uMin = params[startIdx + 1];
        this._uMax = params[startIdx + 2];
    }
}

//-------------------------------------------------------------------------------------------------
