
/*
    Copyright (C) 2016  Torsten Brischalle
    email: torsten@brischalle.de
    web: http://www.aaabbb.de
    
    MIT License

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
*/

//-------------------------------------------------------------------------------------------------

import { Gui } from "../../Framework/Gui/Gui";

//-------------------------------------------------------------------------------------------------

//declare var Flotr : any;
declare var math: any;
declare var MathJax: any;

//-------------------------------------------------------------------------------------------------

export interface ITransferFunction
{
    Function(s: number): number;
}

//-------------------------------------------------------------------------------------------------

class TransferFunction implements ITransferFunction
{
    private _code: any;

    public constructor(formula: string)
    {
        this._code = math.compile(formula);
    }

    public Function(sVal: number): number
    {
        let scope: any = { s: sVal };
        return <number>this._code.eval(scope);
    }
}

//-------------------------------------------------------------------------------------------------

export class ControlPathFormulaSim
{
    //-------------------------------------------------------------------------------------------------

    public constructor()
    {
    }

    //-------------------------------------------------------------------------------------------------

    public ControlPathStepResponse(): void
    {
        try
        {
            let formulaControllerElem: HTMLElement = document.getElementById("formula_controller");
            let formulaControlPathElem: HTMLElement = document.getElementById("formula_control_path");
            let formulaController: string = Gui.GetString("input_controller");
            let formulaControlPath: string = Gui.GetString("input_control_path");

            formulaControllerElem.textContent = "$$" + formulaController + "$$";
            formulaControlPathElem.textContent = "$$" + formulaControlPath + "$$";

            MathJax.Hub.Queue(["Typeset", MathJax.Hub, formulaControllerElem]);
            MathJax.Hub.Queue(["Typeset", MathJax.Hub, formulaControlPathElem]);

            let funcCtrl: TransferFunction = new TransferFunction(formulaController);
            let funcCtrlPath: TransferFunction = new TransferFunction(formulaControlPath);

            let gs: number = ControlPathFormulaSim.FuncClosedLoop(1, funcCtrl, funcCtrlPath);
            console.log(gs);
        }
        catch (err)
        {
            alert(err.message);
        }
    }

    //-------------------------------------------------------------------------------------------------

    public static FuncStep(s: number, stepHeight: number): number
    {
        return stepHeight / s;
    }

    //-------------------------------------------------------------------------------------------------

    public static FuncClosedLoop(s: number, controler: ITransferFunction, controlPath: ITransferFunction): number
    {
        let g: number = controler.Function(s) * controlPath.Function(s);

        return g / (1 + g);
    }

    //-------------------------------------------------------------------------------------------------
}

//-------------------------------------------------------------------------------------------------
