
/*
    Copyright (C) 2016  Torsten Brischalle
    email: torsten@brischalle.de
    web: http://www.aaabbb.de
    
    MIT License

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
*/

//-------------------------------------------------------------------------------------------------

export class Gui
{

    //-------------------------------------------------------------------------------------------------

    public static GetCheckedRadioValue(radioGroupName: string): string
    {
        let radios: NodeListOf<Element> = document.getElementsByName(radioGroupName);
        let i: number;

        for (i = 0; i < radios.length; i++)
        {
            let r: HTMLInputElement = <HTMLInputElement>radios[i];

            if (r.checked)
                return r.value;
        }

        return "";
    }

    //-------------------------------------------------------------------------------------------------

    public static GetComboboxItem(inputId: string): string
    {
        let inputElem: HTMLSelectElement = <HTMLSelectElement>document.getElementById(inputId);

        if (inputElem == null)
            throw new Error("input element not found: " + inputId);

        return inputElem.value;
    }

    //-------------------------------------------------------------------------------------------------

    public static GetString(inputId: string): string
    {
        let inputElem: HTMLInputElement = <HTMLInputElement>document.getElementById(inputId);

        if (inputElem == null)
            throw new Error("input element not found: " + inputId);

        return inputElem.value;
    }

    //-------------------------------------------------------------------------------------------------

    public static Set(inputId: string, val: any): void
    {
        let inputElem: HTMLInputElement = <HTMLInputElement>document.getElementById(inputId);

        if (inputElem)
            inputElem.value = val;
    }

    //-------------------------------------------------------------------------------------------------

    public static GetNumber(inputId: string): number
    {
        let inputElem: HTMLInputElement = <HTMLInputElement>document.getElementById(inputId);

        if (inputElem == null)
            throw new Error("input element not found: " + inputId);

        if (inputElem.value.length == 0)
        {
            throw new Error("input value is not a number");
        }

        return Number(inputElem.value);
    }

    //-------------------------------------------------------------------------------------------------

    public static IsChecked(inputId: string): boolean
    {
        let inputElem: HTMLInputElement = <HTMLInputElement>document.getElementById(inputId);

        if (inputElem == null)
            throw new Error("input element not found: " + inputId);

        return inputElem.checked;
    }

    //-------------------------------------------------------------------------------------------------

    public static IsCheckedWithDefault(inputId: string, defaultVal: boolean): boolean
    {
        let inputElem: HTMLInputElement = <HTMLInputElement>document.getElementById(inputId);

        if (inputElem == null)
            return defaultVal;

        return inputElem.checked;
    }

    //-------------------------------------------------------------------------------------------------

    public static GetTable(tableName: string): HTMLTableElement
    {
        return <HTMLTableElement>document.getElementById(tableName);
    }

    //-------------------------------------------------------------------------------------------------

    public static SetTableCell(tableName: string, row: number, col: number, val: any): void
    {
        this.GetTable(tableName).rows[row].cells[col].textContent = val;
    }

    //-------------------------------------------------------------------------------------------------

    public static ClearTable(table: HTMLTableElement): void
    {
        let rowIdx: number;

        for (rowIdx = 1; rowIdx < table.rows.length; rowIdx++)
        {
            let row: HTMLTableRowElement = <HTMLTableRowElement>table.rows[rowIdx];
            let colIdx: number;

            for (colIdx = 0; colIdx < row.cells.length; colIdx++)
            {
                row.cells[colIdx].textContent = "";
            }
        }
    }

    //-------------------------------------------------------------------------------------------------

    public static SelectClearOptions(selId: string): void
    {
        let sel: HTMLSelectElement = <HTMLSelectElement>document.getElementById(selId);
        let idx: number;

        for (idx = sel.options.length - 1; idx >= 0; idx--)
            sel.remove(idx);
    }

    //-------------------------------------------------------------------------------------------------

    public static SelectAddOptions(selId: string, optList: string[]): void
    {
        let sel: HTMLSelectElement = <HTMLSelectElement>document.getElementById(selId);
        let idx: number;

        for (idx = 0; idx < optList.length; idx++)
        {
            let opt: HTMLOptionElement = <HTMLOptionElement>document.createElement("option");

            opt.innerHTML = optList[idx];
            opt.value = optList[idx];

            sel.appendChild(opt);
        }
    }

    //-------------------------------------------------------------------------------------------------
}
